%
%     Implementation by Piotr Hoownia <holownia@student.agh.edu.pl>
%
%     Copyright (C) 2006-9 by the HeKatE Project
%
%     PlNXT has been develped by the HeKatE Project, 
%     see http://hekate.ia.agh.edu.pl
%
%     This file is part of PlNXT.
%
%     PlNXT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     PlNXT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with PlNXT.  If not, see <http://www.gnu.org/licenses/>.
%

:- module(nxt_actions_simulator,[
				nxt_actions_connection_open/0,
				nxt_actions_connection_close/0,
				nxt_actions_motor_forward/2,
				nxt_actions_motor_backward/2,
				nxt_actions_motor_get_speed/2,
				nxt_actions_motor_stop/1,
				nxt_actions_motor_rotate/3,
				nxt_actions_motor_is_moving/1,
				nxt_actions_touch_sensor/2,
				nxt_actions_sound_sensor/2,
				nxt_actions_light_sensor/2,
				nxt_actions_light_sensor_LED/2,
				nxt_actions_ultrasonic_sensor/2,
				nxt_actions_voltage/1,
				nxt_actions_voltage_millivolt/1,
				nxt_actions_start_program/1,
				nxt_actions_stop_program/0,
				nxt_actions_play_sound_file/2,
				nxt_actions_stop_sound_playback/0,
				nxt_actions_play_tone/2,
				nxt_actions_get_brick_name/1,
				nxt_actions_set_brick_name/1
			       ]).

:- use_module(library(pce)).
:- use_module(threads).

:- dynamic nxt_debugger_item/2.
:- dynamic nxt_debugger_item/3.

/** <module> NXT Mindstroms - Actions Simulator.

Motors: 'A', 'B', 'C'.

@author Piotr Hoownia
@license GNU General Public License
*/

%%	nxt_actions_connection_open
%
%	Opens connection.

nxt_actions_connection_open :-
	
	% Creating main window and dialogs.
	new(MainWindow, frame('PLNXT Debugger')),
	assert(nxt_debugger_item(main_window,MainWindow)),
	
	nxt_debugger_sensors_dialog(DialogSensors),
	
	nxt_debugger_motor_dialog(DialogA, PictureA, SpeedLabelA, 'Motor A'),
	nxt_debugger_motor_dialog(DialogB, PictureB, SpeedLabelB, 'Motor B'),
	nxt_debugger_motor_dialog(DialogC, PictureC, SpeedLabelC, 'Motor C'),
	
	send(MainWindow, append, DialogA),
	send(MainWindow, append, DialogB),
	send(MainWindow, append, DialogC),
	
	assert(nxt_debugger_item(picture,'A',PictureA)),
	assert(nxt_debugger_item(picture,'B',PictureB)),
	assert(nxt_debugger_item(picture,'C',PictureC)),
	assert(nxt_debugger_item(speed_label,'A',SpeedLabelA)),
	assert(nxt_debugger_item(speed_label,'B',SpeedLabelB)),
	assert(nxt_debugger_item(speed_label,'C',SpeedLabelC)),
	
	assert(nxt_debugger_item(speed,'A',0)),
	assert(nxt_debugger_item(speed,'B',0)),
	assert(nxt_debugger_item(speed,'C',0)),
	
	send(DialogC, left, DialogB),
	send(DialogB, below, DialogA),
	send(DialogC, below, DialogA),
	send(DialogSensors, above, DialogA),
				
	send(MainWindow, open),
	writeln('Openning connection...'),
	writeln('Connected (simulator)!').

%%	nxt_actions_connection_close
%
%	Closes connection.

nxt_actions_connection_close :-
	nxt_debugger_item(main_window,MainWindow),
	send(MainWindow,destroy),
	retractall(nxt_debugger_item(_,_)),
	retractall(nxt_debugger_item(_,_,_)),
	writeln('Closing connection...'),
	writeln('Disconnected!').

%%	nxt_actions_motor_forward(+Motor,+Speed).
%
%	Rotates motor forward at specified speed.

nxt_actions_motor_forward(Motor,Speed) :-
	nxt_actions_motor_timer_remove(Motor),
	nxt_debugger_item(picture,Motor,Picture),
	nxt_debugger_item(speed_label,Motor,SpeedLabel),
	picture(forward,Picture),
	retract(nxt_debugger_item(speed,Motor,_)),
	assert(nxt_debugger_item(speed,Motor,Speed)),
	send(SpeedLabel,selection,Speed),
	write('Rotating motor '),
	write(Motor),
	write(' forward at speed: '),
	write(Speed),
	write('.'),nl,!.

%%	nxt_actions_motor_backward(+Motor,+Speed).
%
%	Rotates motor backward at specified speed.

nxt_actions_motor_backward(Motor,Speed) :-
	nxt_actions_motor_timer_remove(Motor),
	nxt_debugger_item(picture,Motor,Picture),
	nxt_debugger_item(speed_label,Motor,SpeedLabel),
	picture(backward,Picture),
	retract(nxt_debugger_item(speed,Motor,_)),
	MinusSpeed is -Speed,
	assert(nxt_debugger_item(speed,Motor,MinusSpeed)),
	send(SpeedLabel,selection,MinusSpeed),
	write('Rotating motor '),
	write(Motor),
	write(' backward at speed: '),
	write(Speed),
	write('.'),nl,!.

%%	nxt_actions_motor_get_speed(+Motor,+Speed).
%
%	Gets speed of the motor.

nxt_actions_motor_get_speed(Motor,Speed) :-
	nxt_debugger_item(speed,Motor,Speed).

%%	nxt_motor_stop(+Motor).
%
%	Stops motor.

nxt_actions_motor_stop(Motor) :-
	nxt_actions_motor_timer_remove(Motor),
	nxt_debugger_item(picture,Motor,Picture),
	nxt_debugger_item(speed_label,Motor,SpeedLabel),
	picture(stopped,Picture),
	retract(nxt_debugger_item(speed,Motor,_)),
	assert(nxt_debugger_item(speed,Motor,0)),
	send(SpeedLabel,selection,0),
	write('Motor '),
	write(Motor),
	write(' has been stopped.'),nl.

%%	nxt_actions_motor_rotate(+Motor,+Speed,+Angle).
%
%	Rotates motor at specified speed.
%	Forward if Angle is positive, backward if Angle is negative.
%	Motor will stop, when specified revolution (Angle in degrees) is reached.

nxt_actions_motor_rotate(Motor,Speed,Angle) :-
	Angle > 0,
	Time is Angle/Speed,
	timer_create(ID,Time,nxt_actions_simulator:nxt_actions_motor_stop(Motor),system),
	nxt_actions_motor_forward(Motor,Speed),
	nxt_actions_motor_timer_change(Motor,ID),!.

nxt_actions_motor_rotate(Motor,Speed,Angle) :-
	Angle < 0,
	Time is -Angle/Speed,
	timer_create(ID,Time,nxt_actions_simulator:nxt_actions_motor_stop(Motor),system),
	nxt_actions_motor_backward(Motor,Speed),
	nxt_actions_motor_timer_change(Motor,ID).

%	nxt_actions_motor_timer_remove(+Motor).
%
%	Removes stopping timer of the Motor.

nxt_actions_motor_timer_remove(Motor) :-
	nxt_debugger_item(timer,Motor,OldID),
	timer_system_kill(OldID),
	retractall(nxt_debugger_item(timer,Motor,_)),!.

nxt_actions_motor_timer_remove(Motor) :-
	% Needs to be retracted (Timer may be fired during the operation
	% and timer_system_kill/1 may fail in previous rule).
       	retractall(nxt_debugger_item(timer,Motor,_)).

%	nxt_actions_motor_timer_change(+Motor,+ID).
%
%	Changes stopping timer of the Motor for a new one with the
%	specified ID.

nxt_actions_motor_timer_change(Motor,ID) :-
	assert(nxt_debugger_item(timer,Motor,ID)).

%%	nxt_actions_motor_is_moving(+Motor).
%
%	Returns true if specified motor is moving. Otherwise fails.

nxt_actions_motor_is_moving(Motor) :-
	nxt_debugger_item(speed,Motor,Speed),
	Speed \= 0.
		
%%	nxt_actions_touch_sensor(+Port,-Value).
%
%	Gets touch sensor reading.
%	Returns 1 if pressed, 0 otherwise.

nxt_actions_touch_sensor(_,Value) :-
	nxt_debugger_item(touch,Touch),
	get(Touch,selection,Value).

%%	nxt_actions_sound_sensor(+Port,-Value).
%
%	Gets sound sensor reading.

nxt_actions_sound_sensor(_,Value) :-
	nxt_debugger_item(sound,Sound),
	get(Sound,selection,Value).

%%	nxt_actions_light_sensor(+Port,-Value).
%
%	Gets light sensor reading.

nxt_actions_light_sensor(_,Value) :-
	nxt_debugger_item(light,Light),
	get(Light,selection,Value).

%%	nxt_actions_light_sensor_LED(+Port,+Setting).
%
%	Sets the LED on if Setting is activate or off if passivate.

nxt_actions_light_sensor_LED(Port,Setting) :-
	write('Light sensor LED connected to port '),
	write(Port),
	write(' has been set to: '),
	write(Setting),
	write(' state. '),nl.

%%	nxt_actions_ultrasonic_sensor(+Port,-Value).
%
%	Gets ultrasonic sensor reading.

nxt_actions_ultrasonic_sensor(_,Value) :-
	nxt_debugger_item(ultrasonic,Ultrasonic),
	get(Ultrasonic,selection,Value).

%%	nxt_actions_voltage(-Voltage).
%
%	Returns battery voltage in V.

nxt_actions_voltage(Voltage) :-
	Voltage is random(9),
	write('Battery Voltage: '),
	write(Voltage),
	write(' V.'),nl.

%%	nxt_actions_voltage_millivolt(-Voltage).
%
%	Returns battery voltage in mV.

nxt_actions_voltage_millivolt(Voltage) :-
	Voltage is random(9000),
	write('Battery Voltage: '),
	write(Voltage),
	write(' mV.'),nl.

%%	nxt_actions_start_program(+File)
%
%	Starts a Lego executable file on the NXT.

nxt_actions_start_program(File) :-
	write('Executing file: '),
	write(File),nl.

%%	nxt_actions_stop_program.
%
%	Stops the currently running Lego executable on the NXT.

nxt_actions_stop_program :-
	writeln('Stopping the currently running program.').

%%	nxt_actions_play_sound_file(+File,+Repeat).
%
%	Plays a sound file from the NXT.
%	Files use the .rso extension.
%	The filename is not case sensitive.
%	nxt_actions_play_sound_file/2 plays sound once if Repeat is true
%	or till nxt_actions_stop_sound_playback is used if Repeat is
%	false.

nxt_actions_play_sound_file(File,true) :-
	write('Playing in loop sound file: '),
	write(File),nl.

nxt_actions_play_sound_file(File,false) :-
	write('Playing once sound file: '),
	write(File),nl.

%%	nxt_actions_stop_sound_playback.
%
%	Stops a sound file that has been playing/repeating.

nxt_actions_stop_sound_playback :-
	writeln('Stopping the currently playing sound.').

%%	nxt_actions_play_tone(Frequency,Duration).
%
%	Plays tone at specified frequency.

nxt_actions_play_tone(Frequency,Duration) :-
	write('Playing tone at frequency: '),
	write(Frequency),
	write(' and duration: '),
	write(Duration),
	write('.'),nl.

%%	nxt_actions_get_brick_name(-Name).
%
%	Gets the name of the brick.

nxt_actions_get_brick_name(Name) :-
	Name='MyBrick',
	write('The name of the brick is: '),
	write(Name),
	write('.'),nl.

%%	nxt_actions_set_brick_name(+Name)
%
%	Sets the name of the brick.

nxt_actions_set_brick_name(Name) :-
	write('Setting the name of the brick to: '),
	write(Name),
	write('.'),nl.



%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	
picture(stopped,Picture) :-
	picture_remove(Picture),
	send(Picture,display,
	     new(Text, text('Stopped')), point(3,24)),
	send(Text, font, font(times, bold, 14)),
	send(Text, colour, colour(red)),!.

picture(forward,Picture) :-
	picture_remove(Picture),
	send(Picture,display,
	     new(Arrow, path(poly, 0, chain(
					    point(30,0),
					    point(0,30),
					    point(20,30),
					    point(20,70),
					    point(40,70),
					    point(40,30),
					    point(60,30),
					    point(30,0)
					   )))),
	send(Arrow, fill_pattern, colour(green)),!.

picture(backward,Picture) :-
	picture_remove(Picture),
	send(Picture,display,
	     new(Arrow, path(poly, 0, chain(
					    point(30,70),
					    point(0,40),
					    point(20,40),
					    point(20,0),
					    point(40,0),
					    point(40,40),
					    point(60,40),
					    point(30,70)
					   )))),
	send(Arrow, fill_pattern, colour(red)),!.

picture_remove(Picture) :-
	get(Picture,member(text),PreviousPicture),
	free(PreviousPicture),!.

picture_remove(Picture) :-
	get(Picture,member(path),PreviousPicture),
	free(PreviousPicture).

nxt_debugger_sensors_dialog(Dialog) :-
	new(Dialog,dialog),
	
	send(Dialog, append,
	     new(Touch, slider(touch, 0, 1, 0))),
	send(Touch, drag, @on),
	send(Touch, attribute, hor_stretch, 100),
	assert(nxt_debugger_item(touch,Touch)),
	
	send(Dialog, append,
	     new(Sound, slider(sound, 0, 1000, 2))),
	send(Sound, drag, @on),
	send(Sound, attribute, hor_stretch, 100),
	assert(nxt_debugger_item(sound,Sound)),
	
	send(Dialog, append,
	     new(Light, slider(light, 0, 1000, 200))),
	send(Light, drag, @on),
	send(Light, attribute, hor_stretch, 100),
	assert(nxt_debugger_item(light,Light)),
	
	send(Dialog, append,
	     new(Ultrasonic, slider(ultrasonic, 0, 255, 50))),
	send(Ultrasonic, drag, @on),
	send(Ultrasonic, attribute, hor_stretch, 100),
	assert(nxt_debugger_item(ultrasonic,Ultrasonic)).

nxt_debugger_motor_dialog(Dialog, Picture, Speed, Label) :-
	new(Dialog, dialog),
	send(Dialog, append,
	     label(label, Label)),
	
	send(Dialog, append, new(Picture, picture(size := size(62,73)))),
	send(Picture,display,
	     new(Text, text('Stopped')), point(3,24)),
	send(Text, font, font(times, bold, 14)),
	send(Text, colour, colour(red)),
	
	send(Dialog, append, new(Speed, int_item(speed))),
	send(Speed, length, 8),
	send(Speed, editable, @off),
	send(Speed, selection, 0),
	send(Speed, below, Picture).




















