%
%     Implementation by Piotr Hoownia <holownia@student.agh.edu.pl>
%
%     Copyright (C) 2006-9 by the HeKatE Project
%
%     PLNXT has been develped by the HeKatE Project, 
%     see http://hekate.ia.agh.edu.pl
%
%     This file is part of PLNXT.
%
%     PLNXT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     PLNXT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with PLNXT.  If not, see <http://www.gnu.org/licenses/>.
%

:- module(nxt_movement,[
			nxt_open/0,
			nxt_close/0,
			nxt_set_robot/10,
			nxt_stop/0,
			nxt_is_stopped/0,
			nxt_go/1, nxt_go/2, nxt_go/3,
			nxt_go_sec/2, nxt_go_sec/3,
			nxt_go_cm/2, nxt_go_cm/3,
			nxt_go_cm_sec/2, nxt_go_cm_sec/3,
			nxt_turn/2, nxt_turn/3, nxt_turn/4,
			nxt_rotate/2, nxt_rotate/3,
			nxt_pincer/1, nxt_pincer/2, nxt_pincer/3,
			nxt_touch/1, nxt_touch/2,
			nxt_sound/1, nxt_sound/2,
			nxt_light/1, nxt_light/2,
			nxt_light_LED/1, nxt_light_LED/2,
			nxt_ultrasonic/1, nxt_ultrasonic/2
		       ]).

/** <module> NXT Mindstroms - simple movement.

@author Piotr Hoownia
@license GNU General Public License
*/

:- dynamic nxt_robot/10.
	
:- use_module(nxt_sensomoto).
:- use_module(threads).


%%	nxt_set_robot(+WheelCircumference,+AxleLenght,+LeftMotor,+RightMotor,+PincerMotor,+Reverse,+TouchPort,+SoundPort,+LightPort,+UltrasonicPort).
%
%	Changes the robot's settings. !! Reverse not implemented!
%	Set sensor port to 'S0' if there is no sensor.

nxt_set_robot(WheelCircumference,AxleLenght,LeftMotor,RightMotor,PincerMotor,
	      Reverse,TouchPort,SoundPort,LightPort,UltrasonicPort) :-
	nonvar(WheelCircumference),nonvar(AxleLenght),
	nonvar(LeftMotor),nonvar(RightMotor),nonvar(PincerMotor),nonvar(Reverse),
	retractall(nxt_robot(_,_,_,_,_,_,_,_,_,_)),
	assert(nxt_robot(WheelCircumference,AxleLenght,LeftMotor,RightMotor,PincerMotor,
			 Reverse,TouchPort,SoundPort,LightPort,UltrasonicPort)).

:- nxt_set_robot(17.5,11,'C','B','A',false,'S1','S2','S3','S4').


%%	nxt_open.
%
%	Opens connection.

nxt_open :-
	nxt_connection_open,
	% First sensors readings are wrong.
	nxt_ultrasonic(_,force),
	nxt_light(_,force),
	nxt_sound(_,force),
	nxt_touch(_,force),
	nxt_light_LED(passivate,force),
	nxt_robot(_,_,LM,RM,_,_,_,_,_,_),
	% This may be needed after reconnection.
	nxt_motor_reset_position(LM,last),
	nxt_motor_reset_position(RM,last).


%%	nxt_close.
%
%	Closes connection.

nxt_close :-
	trigger_killall,
	timer_killall,
	nxt_connection_close.


%%	nxt_stop.
%
%	Stops the robot.

nxt_stop :-
	nxt_robot(_,_,LM,RM,_,_,_,_,_,_),
	nxt_motor_sync(LM,RM,0).


%%	nxt_is_stopped.
%
%	Returns true if robot is stopped. Otherwise fails.

nxt_is_stopped :-
	nxt_robot(_,_,LM,RM,PM,_,_,_,_,_),
	not(nxt_motor_is_moving(LM)),
	not(nxt_motor_is_moving(RM)),
	not(nxt_motor_is_moving(PM)).


%%	nxt_go(+Speed).
%
%	Moves the robot forward (if Speed is greater than 0)
%	or backward (if Speed is smaller than 0).
%	Speed is the rotational speed of wheels in degrees per second.
%	Wheels stop when nxt_stop predicate is called.
%	Starts when the robot is stopped.

nxt_go(Speed) :-
	wait_till(nxt_is_stopped),
	nxt_go(Speed,force).


%%	nxt_go(+Speed,+Option).
%
%	The same as nxt_go(+Speed), except it works immediately.
%	Option is:
%
%		* force

nxt_go(Speed,force) :-
	nxt_robot(_,_,LM,RM,_,_,_,_,_,_),
	nxt_motor_sync(LM,RM,Speed,0).


%%	nxt_go(+Speed,+Angle).
%
%	Moves the robot forward (if Speed is greater than 0)
%	or backward (if Speed is smaller than 0).
%	Speed is the rotational speed of wheels in degrees per second.
%	Wheels stop after revolution of Angle (in degrees).
%	Starts when the robot is stopped.

nxt_go(Speed,Angle) :-
	wait_till(nxt_is_stopped),
	nxt_go(Speed,Angle,force).


%%	nxt_go(+Speed,+Angle,+Option).
%
%	The same as nxt_go(+Speed,+Angle), except it works immediately.
%	Option is:
%
%		* force

nxt_go(Speed,Angle,force) :-
	nxt_robot(_,_,LM,RM,_,_,_,_,_,_),
	nxt_motor_sync(LM,RM,Speed,0,Angle).


%%	nxt_go_sec(+Speed,+Time).
%
%	Moves the robot forward (if Speed is greater than 0)
%	or backward (if Speed is smaller than 0).
%	Speed is the rotational speed of wheels in degrees per second.
%	Wheels stop after specified time (in seconds).
%	Starts when the robot is stopped.

nxt_go_sec(Speed,Time) :-
	wait_till(nxt_is_stopped),
	nxt_go_sec(Speed,Time,force).


%%	nxt_go_sec(+Speed,+Time,+Option).
%
%	The same as nxt_go_sec(+Speed,+Time), except it works immediately.
%	Option is:
%
%		* force

nxt_go_sec(Speed,Time,force) :-
	nxt_robot(_,_,LM,RM,_,_,_,_,_,_),
	nxt_motor_sync(LM,RM,Speed,0,time(Time)).


%%	nxt_go_cm(+Speed,+Distance).
%
%	Moves the robot forward (if Speed is greater than 0)
%	or backward (if Speed is smaller than 0).
%	Speed is the rotational speed of wheels in degrees per second.
%	Wheels will stop, if the Distance (in cm) is reached.
%	Starts when the robot is stopped.

nxt_go_cm(Speed,Distance) :-
	wait_till(nxt_is_stopped),
	nxt_go_cm(Speed,Distance,force).


%%	nxt_go_cm(+Speed,+Distance,+Option).
%
%	The same as nxt_go_cm(+Speed,+Distance), except it works immediately.
%	Option is:
%
%		* force

nxt_go_cm(Speed,Distance,force) :-
	nxt_robot(WC,_,_,_,_,_,_,_,_,_),
	Angle is round(Distance/WC*360),
	nxt_go(Speed,Angle).


%%	nxt_go_cm_sec(+Distance,+Time).
%
%	Moves the robot forward (if Distance is greater than 0)
%	or backward (if Distance is smaller than 0).
%	Robot reaches the Distance (in cm) in Time (in seconds).
%	Starts when the robot is stopped.

nxt_go_cm_sec(Distance,Time) :-
	wait_till(nxt_is_stopped),
	nxt_go_cm_sec(Distance,Time,force).


%%	nxt_go_cm_sec(+Distance,+Time,+Option).
%
%	The same as nxt_go_cm_sec(+Distance,+Time), except it works immediately.
%	Option is:
%
%		* force

nxt_go_cm_sec(Distance,Time,force) :-
	Distance > 0,
	nxt_robot(WC,_,_,_,_,_,_,_,_,_),
	Angle is round(Distance/WC*360),
	Speed is round(Angle/Time),
	nxt_go(Speed,Angle,force).

nxt_go_cm_sec(Distance,Time,force) :-
	Distance < 0,
	nxt_robot(WC,_,_,_,_,_,_,_,_,_),
	Angle is round(Distance/WC*360),
	Speed is -round(Angle/Time),
	nxt_go(Speed,Angle,force).
 %%	nxt_rotate(+Speed,+Degrees).
%
%	Rotates the robot in place to its right (if Degrees is greater than 0)
%	or left (if Degrees is smaller than 0).
%	Speed is the rotational speed of wheels in degrees per second.
%	Wheels will stop, when specified revolution (Degrees) of the robot is reached.
%	Starts when the robot is stopped.

nxt_rotate(Speed,Degrees) :-
	wait_till(nxt_is_stopped),
	nxt_rotate(Speed,Degrees,force).


%%	nxt_rotate(+Speed,+Degrees,+Option).
%
%	The same as nxt_rotate(+Speed,+Degrees), except it works immediately.
%	Option is:
%
%		* force

nxt_rotate(Speed,Degrees,force) :-
	nxt_robot(WC,AL,_,_,_,_,_,_,_,_),
	%Distance is pi*AL*Degrees/360,
	%Angle is Distance/WC*360,
	Angle is round(pi*AL*Degrees/WC),
	nxt_turn(Speed,Angle,force).


%%	nxt_turn(+Speed,+Angle).
%
%	Rotates the robot in place to its right (if Angle is greater than 0)
%	or left (if Angle is smaller than 0).
%	Speed is the rotational speed of wheels in degrees per second.
%	Wheels stop after revolution of Angle (in degrees).
%	Starts when the robot is stopped.

nxt_turn(Speed,Angle) :-
	wait_till(nxt_is_stopped),
	nxt_turn(Speed,Angle,force).


%%	nxt_turn(+Speed,+Angle,+Option).
%
%	The same as nxt_turn(+Speed,+Angle), except it works immediately.
%	Option is:
%
%		* force

nxt_turn(Speed,Angle,force) :-
	Angle > 0,
	nxt_robot(_,_,LM,RM,_,_,_,_,_,_),
	nxt_motor_sync(LM,RM,Speed,100,Angle).

nxt_turn(Speed,Angle,force) :-
	Angle < 0,
	nxt_robot(_,_,LM,RM,_,_,_,_,_,_),
	MinusAngle is -Angle,
	nxt_motor_sync(LM,RM,Speed,-100,MinusAngle).


%%	nxt_turn(+Speed,+Degrees,+Radius).
%
%	Makes robot turn with specified turning radius (Radius)
%	moving forward (if Speed is positive) or backward (if negative).
%	Robot turns right (if Degrees is positive) or left (if negative).
%	Robot reaches the specified revolution (Degrees) and stops.
%	Speed is a speed of an outer wheel.

nxt_turn(Speed,Degrees,Radius) :-
	wait_till(nxt_is_stopped),
	nxt_turn(Speed,Degrees,Radius,force).


%%	nxt_turn(+Speed,+Degrees,+Radius,+Option).
%
%	The same as nxt_turn(+Radius,+Degrees,+Time), except it works immediately.
%	Option is:
%
%		* force

nxt_turn(Speed,Degrees,Radius,force) :-
	Degrees > 0,
	Speed > 0,
	%RF
	nxt_turn_parameters(Degrees,Radius,Turn_ratio,Angle),
	nxt_robot(_,_,LM,RM,_,_,_,_,_,_),
	nxt_motor_sync(LM,RM,Speed,Turn_ratio,angle(Angle)).

nxt_turn(Speed,Degrees,Radius,force) :-
	Degrees < 0,
	Speed > 0,
	%LF
	nxt_turn_parameters(-Degrees,Radius,Turn_ratio,Angle),
	nxt_robot(_,_,LM,RM,_,_,_,_,_,_),
	nxt_motor_sync(LM,RM,Speed,-Turn_ratio,angle(Angle)).

nxt_turn(Speed,Degrees,Radius,force) :-
	Degrees > 0,
	Speed < 0,
	%RB
	nxt_turn_parameters(Degrees,Radius,Turn_ratio,Angle),
	nxt_robot(_,_,LM,RM,_,_,_,_,_,_),
	nxt_motor_sync(LM,RM,Speed,-Turn_ratio,angle(Angle)).

nxt_turn(Speed,Degrees,Radius,force) :-
	Degrees < 0,
	Speed < 0,
	%LB
	nxt_turn_parameters(-Degrees,Radius,Turn_ratio,Angle),
	nxt_robot(_,_,LM,RM,_,_,_,_,_,_),
	nxt_motor_sync(LM,RM,Speed,Turn_ratio,angle(Angle)).

nxt_turn_parameters(Degrees,Radius,Turn_ratio,OutAngle) :-
	nxt_robot(WC,AL,_,_,_,_,_,_,_,_),
	%InDistance is 2*pi*Radius*Degrees/360,
	%OutDistance is 2*pi*(Radius+AL)*Degrees/360,
	%InAngle is InDistance/WC*360,
	%OutAngle is OutDistance/WC*360,
	InAngle is round(2*pi*Radius*Degrees/WC),
	OutAngle is round(2*pi*(Radius+AL)*Degrees/WC),
	Turn_ratio is round(37 - ((37*InAngle)/OutAngle)).


%%	nxt_pincer(+Option).
%
%	Moves pincer when a robot is stopped.
%	Option is one of:
%
%		* open
%
%		* close

nxt_pincer(open) :-
	nxt_pincer(open,300).

nxt_pincer(close) :-
	nxt_pincer(close,300).


%%	nxt_pincer(+Option1,+Option2).
%
%	Moves pincer immediately.
%	Option is one of:
%
%		* open
%
%		* close
%
%	Option2 is:
%
%		* force

nxt_pincer(open,force) :-
	nxt_pincer(open,300,force).

nxt_pincer(close,force) :-
	nxt_pincer(close,300,force).


%%	nxt_pincer(+Option,+Speed).
%
%	Moves pincer when a robot is stopped with specified rotational motor speed.
%	Option is one of:
%
%		* open
%
%		* close

nxt_pincer(open,Speed) :-
	wait_till(nxt_is_stopped),
	nxt_pincer(open,Speed,force).

nxt_pincer(close,Speed) :-
	wait_till(nxt_is_stopped),
	nxt_pincer(close,Speed,force).


%%	nxt_pincer(+Option1,+Speed,+Option2).
%
%	Moves pincer immediately with specified rotational motor speed.
%	Option1 is one of:
%
%		* open
%
%		* close
%
%	Option2 is:
%
%		* force

nxt_pincer(open,Speed,force) :-
	nxt_robot(_,_,_,_,PM,_,_,_,_,_),
	nxt_motor(PM,Speed,90).

nxt_pincer(close,Speed,force) :-
	nxt_robot(_,_,_,_,PM,_,_,_,_,_),
	nxt_motor(PM,-Speed,90).
%FIXME Dodac sprawdzanie pozycji silnika, zeby np. nie otwierac dwa razy pod rzad szczypiec.



%%	nxt_touch(-Value).
%
%	Gets touch sensor reading.
%	Returns 1 if pressed, 0 otherwise.
%	Starts when the robot is stopped.

nxt_touch(Value) :-
	wait_till(nxt_is_stopped),
	nxt_touch(Value,force).


%%	nxt_touch(-Value,+Option).
%
%	The same as nxt_touch(-Value), except it works immediately.
%	Option is:
%
%		* force

nxt_touch(Value,force) :-
	nxt_robot(_,_,_,_,_,_,'S0',_,_,_),
	Value='error'.

nxt_touch(Value,force) :-
	nxt_robot(_,_,_,_,_,_,Port,_,_,_),
	nxt_touch_sensor(Port,Value).


%%	nxt_sound(-Value).
%
%	Gets sound sensor reading (0-100).
%	Starts when the robot is stopped.

nxt_sound(Value) :-	
	wait_till(nxt_is_stopped),
	nxt_sound(Value,force).


%%	nxt_sound(-Value,+Option).
%
%	The same as nxt_sound(-Value), except it works immediately.
%	Option is:
%
%		* force

nxt_sound(Value,force) :-
	nxt_robot(_,_,_,_,_,_,_,'S0',_,_),
	Value='error'.

nxt_sound(Value,force) :-
	nxt_robot(_,_,_,_,_,_,_,Port,_,_),
	nxt_sound_sensor(Port,Value).


%%	nxt_light(-Value).
%
%	Gets light sensor reading (0-100).
%	Starts when the robot is stopped.

nxt_light(Value) :-
	wait_till(nxt_is_stopped),
	nxt_light(Value,force).


%%	nxt_light(-Value,+Option).
%
%	The same as nxt_light(-Value), except it works immediately.
%	Option is:
%
%		* force

nxt_light(Value,force) :-
	nxt_robot(_,_,_,_,_,_,_,_,'S0',_),
	Value='error'.

nxt_light(Value,force) :-
	nxt_robot(_,_,_,_,_,_,_,_,Port,_),
	nxt_light_sensor(Port,Value).


%%	nxt_light_LED(+Setting).
%
%	Sets the LED on if Setting is =activate= or off if =passivate=.
%	Starts when the robot is stopped.

nxt_light_LED(Setting) :-
	wait_till(nxt_is_stopped),
	nxt_light_LED(Setting,force).


%%	nxt_light_LED(+Setting,+Option).
%
%	The same as nxt_light_LED(+Setting), except it works immediately.
%	Option is:
%
%		* force

nxt_light_LED(_,force) :-
	nxt_robot(_,_,_,_,_,_,_,_,'S0',_).

nxt_light_LED(Setting,force) :-
	nxt_robot(_,_,_,_,_,_,_,_,Port,_),
	nxt_light_sensor_LED(Port,Setting).	


%%	nxt_ultrasonic(-Value).
%
%	Gets ultrasonic sensor reading (0-255).
%	Starts when the robot is stopped.

nxt_ultrasonic(Value) :-
	wait_till(nxt_is_stopped),
	nxt_ultrasonic(Value,force).


%%	nxt_ultrasonic(-Value,+Option).
%
%	The same as nxt_ultrasonic(-Value), except it works immediately.
%	Option is:
%
%		* force

nxt_ultrasonic(Value,force) :-
	nxt_robot(_,_,_,_,_,_,_,_,_,'S0'),
	Value='error'.

nxt_ultrasonic(Value,force) :-
	nxt_robot(_,_,_,_,_,_,_,_,_,Port),
	nxt_ultrasonic_sensor(Port,Value).

