% Threads testing. Use test_go/0 to start.

:- use_module('../lib/threads').

:- dynamic test_value/1.

test_go :-
	%test_wait_till.
	%test_triggers.
	test_timers.

test_wait_till :-
	writeln('First line'),
	timer_create(_,5,assert(test_value(100))),
	wait_till(test_value(100)),
	writeln('Second line'),
	timer_create(_,5,assert(test_value(101))),
	wait_till(test_value(101)),
	writeln('Third line'),
	timer_create(_,5,assert(test_value(102))),
	wait_till(test_value(102)),
	writeln('Fourth line').

test_triggers :-
	writeln('@@@@@@'), writeln('TESTING TRIGGERS:'), writeln('@@@@@@'), nl,
	test_trigger_create(_,1),
	go_on,
	test_trigger_create(_,2,3),
	go_on,
	test_trigger_set_event(1),
	go_on,
	test_trigger_set_event(2),
	go_on,
	test_trigger_unset_events.

test_timers :-
	writeln('@@@@@@'), writeln('TESTING TIMERS:'), writeln('@@@@@@'), nl,
	test_timer_create(_,10),
	go_on,
	test_timer_create(ID2,20,system),
	go_on,
	test_timer_create(_,30),
	go_on,
	test_timer_system_kill(ID2),
	go_on,
	test_timer_killall.

go_on :-
	writeln('Press any key to continue...'),
	get_single_char(_),
	nl.

test_write_triggers :-
	trigger_exists(ID),
	write(ID), write(' '),
	fail.

test_write_triggers.

test_write_timers :-
	timer_exists(ID),
	write(ID), write(' '),
	fail.

test_write_timers.

test_write_threads :-
	thread_property(ID,status(running)),
	write(ID), write(' '),
	fail.

test_write_threads.

test_state :-
	write('All running triggers: '), test_write_triggers, nl,
	write('All running timers: '), test_write_timers, nl,
	write('All running threads: '), test_write_threads, nl,
	nl.

test_trigger_create(ID,TV) :-
	trigger_create(ID,test_value(TV),test_trigger_fire),
	write('The trigger with the ID '), write(ID), write(' has been created.'), nl,
	nl,
	test_state.

test_trigger_create(ID,TV,Counter) :-
	trigger_create(ID,test_value(TV),test_trigger_fire,Counter),
	write('The trigger with the ID '), write(ID), write(' has been created.'), nl,
	nl,
	test_state.

test_trigger_fire :-
	thread_self(ID),
	write('Trigger with ID '), write(ID), write(' has been fired.'), nl,
	nl,
	test_state.

test_trigger_set_event(TV) :-
	assert(test_value(TV)).

test_trigger_unset_events :-
	retractall(test_value(_)).

test_trigger_unset_events.

test_timer_create(ID,Time) :-
	timer_create(ID,Time,test_timer_fire),
	write('The timer with the ID '), write(ID), write(' has been created.'), nl,
	write('	It should be fired in '), write(Time), write(' seconds.'), nl,
	nl,
	test_state.

test_timer_create(ID,Time,system) :-
	timer_create(ID,Time,test_timer_fire,system),
	write('The system timer with the ID '), write(ID), write(' has been created.'), nl,
	write('	It should be fired in '), write(Time), write(' seconds.'), nl,
	nl,
	test_state.

test_timer_fire :-
	thread_self(ID),
	write('Timer with ID '), write(ID), write(' has been fired.'), nl,
	nl,
	test_state.

test_timer_kill(ID) :-
	timer_kill(ID),
	write('Timer with ID '), write(ID), write(' has been killed.'),nl,
	test_state.

test_timer_system_kill(ID) :-
	timer_system_kill(ID),
	write('Timer with ID '), write(ID), write(' has been killed.'),nl,
	test_state.

test_timer_killall :-
	timer_killall,
	writeln('All timers has been killed.'),
	test_state.

:- writeln('Threads testing. Use test_go/0 to start.').