package icommand.vision;

import java.awt.Dimension;

import javax.media.Buffer;
import javax.media.format.RGBFormat;

/**
 * Title: Big fonts Description: Support for overlaying text on video streams
 * 
 * @author Konrad Rzeszutek <konrad@darnok.org>
 */
public class Font
{

   public static final int FONT_8x8 = 1;
   public static final int FONT_6x11 = 10;

   private Font ()
   {}

   /**
    * Write text over the RGB image
    * 
    * @param mesg the text to write
    * @param font the big font to use
    * @param x the x coordinate of the bottom left corner
    * @param y the y coordinate of the bottom left corner
    * @param r the red value of the RGB color of the text
    * @param g the green value of the RGB color of the text
    * @param b the blue value of the RGB color of the text
    * @param inBuffer the video Buffer to overlay the text over
    */
   public static void println (String mesg, int font, int x, int y, byte r,
      byte g, byte b, Buffer inBuffer)
   {

      byte[] outData = (byte[]) inBuffer.getData();
      RGBFormat vfIn = (RGBFormat) inBuffer.getFormat();
      Dimension sizeIn = vfIn.getSize();

      int pixStrideIn = vfIn.getPixelStride();
      int lineStrideIn = vfIn.getLineStride();
      int f = 0;
      int op = 0;
      int xx, yy;
      int CHAR_HEIGHT = 11;
      int CHAR_LENGTH = 7;
      int CHAR_START;
      int c;
      char[] string;
      int max;
      int[] fonts;

      fonts = font_6x11;
      if (font == FONT_8x8)
      {
         CHAR_HEIGHT = CHAR_LENGTH = 8;
         fonts = font_8x8;
      }
      string = mesg.toCharArray();
      max = string.length;

      if (max * pixStrideIn * CHAR_LENGTH > lineStrideIn)
         max = lineStrideIn;
      if (y < CHAR_HEIGHT)
         y = CHAR_HEIGHT;
      if (y > sizeIn.height - CHAR_HEIGHT)
         y = sizeIn.height - CHAR_HEIGHT;
      for (int ii = 0; ii < string.length; ii++)
      {
         c = (byte) string[ii];
         CHAR_START = 8; // Magic number - ie, There are only 8 bits in a byte
                         // and
         // we iterate thru them.
         for (xx = x + (CHAR_LENGTH * ii); xx < x + (CHAR_LENGTH * ii)
            + CHAR_LENGTH; xx++)
         {
            for (yy = CHAR_HEIGHT - 1; yy >= 0; yy--)
            {
               f = fonts[(CHAR_HEIGHT * c) + yy];
               op = lineStrideIn * (y - yy) + xx * pixStrideIn;
               if ((f & (1 << CHAR_START)) > 0)
               {
                  outData[op++] = b;
                  outData[op++] = g;
                  outData[op++] = r;
               }
            }
            CHAR_START--;
         }
      }

   }

   private static final int[] font_8x8 =
   {
      /* 0 0x00 '^@' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 1 0x01 '^A' */
      0x7e, /* 01111110 */
      0x81, /* 10000001 */
      0xa5, /* 10100101 */
      0x81, /* 10000001 */
      0xbd, /* 10111101 */
      0x99, /* 10011001 */
      0x81, /* 10000001 */
      0x7e, /* 01111110 */

      /* 2 0x02 '^B' */
      0x7e, /* 01111110 */
      0xff, /* 11111111 */
      0xdb, /* 11011011 */
      0xff, /* 11111111 */
      0xc3, /* 11000011 */
      0xe7, /* 11100111 */
      0xff, /* 11111111 */
      0x7e, /* 01111110 */

      /* 3 0x03 '^C' */
      0x6c, /* 01101100 */
      0xfe, /* 11111110 */
      0xfe, /* 11111110 */
      0xfe, /* 11111110 */
      0x7c, /* 01111100 */
      0x38, /* 00111000 */
      0x10, /* 00010000 */
      0x00, /* 00000000 */

      /* 4 0x04 '^D' */
      0x10, /* 00010000 */
      0x38, /* 00111000 */
      0x7c, /* 01111100 */
      0xfe, /* 11111110 */
      0x7c, /* 01111100 */
      0x38, /* 00111000 */
      0x10, /* 00010000 */
      0x00, /* 00000000 */

      /* 5 0x05 '^E' */
      0x38, /* 00111000 */
      0x7c, /* 01111100 */
      0x38, /* 00111000 */
      0xfe, /* 11111110 */
      0xfe, /* 11111110 */
      0xd6, /* 11010110 */
      0x10, /* 00010000 */
      0x38, /* 00111000 */

      /* 6 0x06 '^F' */
      0x10, /* 00010000 */
      0x38, /* 00111000 */
      0x7c, /* 01111100 */
      0xfe, /* 11111110 */
      0xfe, /* 11111110 */
      0x7c, /* 01111100 */
      0x10, /* 00010000 */
      0x38, /* 00111000 */

      /* 7 0x07 '^G' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x3c, /* 00111100 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 8 0x08 '^H' */
      0xff, /* 11111111 */
      0xff, /* 11111111 */
      0xe7, /* 11100111 */
      0xc3, /* 11000011 */
      0xc3, /* 11000011 */
      0xe7, /* 11100111 */
      0xff, /* 11111111 */
      0xff, /* 11111111 */

      /* 9 0x09 '^I' */
      0x00, /* 00000000 */
      0x3c, /* 00111100 */
      0x66, /* 01100110 */
      0x42, /* 01000010 */
      0x42, /* 01000010 */
      0x66, /* 01100110 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */

      /* 10 0x0a '^J' */
      0xff, /* 11111111 */
      0xc3, /* 11000011 */
      0x99, /* 10011001 */
      0xbd, /* 10111101 */
      0xbd, /* 10111101 */
      0x99, /* 10011001 */
      0xc3, /* 11000011 */
      0xff, /* 11111111 */

      /* 11 0x0b '^K' */
      0x0f, /* 00001111 */
      0x07, /* 00000111 */
      0x0f, /* 00001111 */
      0x7d, /* 01111101 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0x78, /* 01111000 */

      /* 12 0x0c '^L' */
      0x3c, /* 00111100 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x3c, /* 00111100 */
      0x18, /* 00011000 */
      0x7e, /* 01111110 */
      0x18, /* 00011000 */

      /* 13 0x0d '^M' */
      0x3f, /* 00111111 */
      0x33, /* 00110011 */
      0x3f, /* 00111111 */
      0x30, /* 00110000 */
      0x30, /* 00110000 */
      0x70, /* 01110000 */
      0xf0, /* 11110000 */
      0xe0, /* 11100000 */

      /* 14 0x0e '^N' */
      0x7f, /* 01111111 */
      0x63, /* 01100011 */
      0x7f, /* 01111111 */
      0x63, /* 01100011 */
      0x63, /* 01100011 */
      0x67, /* 01100111 */
      0xe6, /* 11100110 */
      0xc0, /* 11000000 */

      /* 15 0x0f '^O' */
      0x18, /* 00011000 */
      0xdb, /* 11011011 */
      0x3c, /* 00111100 */
      0xe7, /* 11100111 */
      0xe7, /* 11100111 */
      0x3c, /* 00111100 */
      0xdb, /* 11011011 */
      0x18, /* 00011000 */

      /* 16 0x10 '^P' */
      0x80, /* 10000000 */
      0xe0, /* 11100000 */
      0xf8, /* 11111000 */
      0xfe, /* 11111110 */
      0xf8, /* 11111000 */
      0xe0, /* 11100000 */
      0x80, /* 10000000 */
      0x00, /* 00000000 */

      /* 17 0x11 '^Q' */
      0x02, /* 00000010 */
      0x0e, /* 00001110 */
      0x3e, /* 00111110 */
      0xfe, /* 11111110 */
      0x3e, /* 00111110 */
      0x0e, /* 00001110 */
      0x02, /* 00000010 */
      0x00, /* 00000000 */

      /* 18 0x12 '^R' */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x7e, /* 01111110 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x7e, /* 01111110 */
      0x3c, /* 00111100 */
      0x18, /* 00011000 */

      /* 19 0x13 '^S' */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x00, /* 00000000 */
      0x66, /* 01100110 */
      0x00, /* 00000000 */

      /* 20 0x14 '^T' */
      0x7f, /* 01111111 */
      0xdb, /* 11011011 */
      0xdb, /* 11011011 */
      0x7b, /* 01111011 */
      0x1b, /* 00011011 */
      0x1b, /* 00011011 */
      0x1b, /* 00011011 */
      0x00, /* 00000000 */

      /* 21 0x15 '^U' */
      0x3e, /* 00111110 */
      0x61, /* 01100001 */
      0x3c, /* 00111100 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x3c, /* 00111100 */
      0x86, /* 10000110 */
      0x7c, /* 01111100 */

      /* 22 0x16 '^V' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7e, /* 01111110 */
      0x7e, /* 01111110 */
      0x7e, /* 01111110 */
      0x00, /* 00000000 */

      /* 23 0x17 '^W' */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x7e, /* 01111110 */
      0x18, /* 00011000 */
      0x7e, /* 01111110 */
      0x3c, /* 00111100 */
      0x18, /* 00011000 */
      0xff, /* 11111111 */

      /* 24 0x18 '^X' */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x7e, /* 01111110 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */

      /* 25 0x19 '^Y' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x7e, /* 01111110 */
      0x3c, /* 00111100 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */

      /* 26 0x1a '^Z' */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x0c, /* 00001100 */
      0xfe, /* 11111110 */
      0x0c, /* 00001100 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 27 0x1b '^[' */
      0x00, /* 00000000 */
      0x30, /* 00110000 */
      0x60, /* 01100000 */
      0xfe, /* 11111110 */
      0x60, /* 01100000 */
      0x30, /* 00110000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 28 0x1c '^\' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xc0, /* 11000000 */
      0xc0, /* 11000000 */
      0xc0, /* 11000000 */
      0xfe, /* 11111110 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 29 0x1d '^]' */
      0x00, /* 00000000 */
      0x24, /* 00100100 */
      0x66, /* 01100110 */
      0xff, /* 11111111 */
      0x66, /* 01100110 */
      0x24, /* 00100100 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 30 0x1e '^^' */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x7e, /* 01111110 */
      0xff, /* 11111111 */
      0xff, /* 11111111 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 31 0x1f '^_' */
      0x00, /* 00000000 */
      0xff, /* 11111111 */
      0xff, /* 11111111 */
      0x7e, /* 01111110 */
      0x3c, /* 00111100 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 32 0x20 ' ' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 33 0x21 '!' */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x3c, /* 00111100 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */

      /* 34 0x22 '"' */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x24, /* 00100100 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 35 0x23 '#' */
      0x6c, /* 01101100 */
      0x6c, /* 01101100 */
      0xfe, /* 11111110 */
      0x6c, /* 01101100 */
      0xfe, /* 11111110 */
      0x6c, /* 01101100 */
      0x6c, /* 01101100 */
      0x00, /* 00000000 */

      /* 36 0x24 '$' */
      0x18, /* 00011000 */
      0x3e, /* 00111110 */
      0x60, /* 01100000 */
      0x3c, /* 00111100 */
      0x06, /* 00000110 */
      0x7c, /* 01111100 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */

      /* 37 0x25 '%' */
      0x00, /* 00000000 */
      0xc6, /* 11000110 */
      0xcc, /* 11001100 */
      0x18, /* 00011000 */
      0x30, /* 00110000 */
      0x66, /* 01100110 */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */

      /* 38 0x26 '&' */
      0x38, /* 00111000 */
      0x6c, /* 01101100 */
      0x38, /* 00111000 */
      0x76, /* 01110110 */
      0xdc, /* 11011100 */
      0xcc, /* 11001100 */
      0x76, /* 01110110 */
      0x00, /* 00000000 */

      /* 39 0x27 ''' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x30, /* 00110000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 40 0x28 '(' */
      0x0c, /* 00001100 */
      0x18, /* 00011000 */
      0x30, /* 00110000 */
      0x30, /* 00110000 */
      0x30, /* 00110000 */
      0x18, /* 00011000 */
      0x0c, /* 00001100 */
      0x00, /* 00000000 */

      /* 41 0x29 ')' */
      0x30, /* 00110000 */
      0x18, /* 00011000 */
      0x0c, /* 00001100 */
      0x0c, /* 00001100 */
      0x0c, /* 00001100 */
      0x18, /* 00011000 */
      0x30, /* 00110000 */
      0x00, /* 00000000 */

      /* 42 0x2a '*' */
      0x00, /* 00000000 */
      0x66, /* 01100110 */
      0x3c, /* 00111100 */
      0xff, /* 11111111 */
      0x3c, /* 00111100 */
      0x66, /* 01100110 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 43 0x2b '+' */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x7e, /* 01111110 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 44 0x2c ',' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x30, /* 00110000 */

      /* 45 0x2d '-' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7e, /* 01111110 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 46 0x2e '.' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */

      /* 47 0x2f '/' */
      0x06, /* 00000110 */
      0x0c, /* 00001100 */
      0x18, /* 00011000 */
      0x30, /* 00110000 */
      0x60, /* 01100000 */
      0xc0, /* 11000000 */
      0x80, /* 10000000 */
      0x00, /* 00000000 */

      /* 48 0x30 '0' */
      0x38, /* 00111000 */
      0x6c, /* 01101100 */
      0xc6, /* 11000110 */
      0xd6, /* 11010110 */
      0xc6, /* 11000110 */
      0x6c, /* 01101100 */
      0x38, /* 00111000 */
      0x00, /* 00000000 */

      /* 49 0x31 '1' */
      0x18, /* 00011000 */
      0x38, /* 00111000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x7e, /* 01111110 */
      0x00, /* 00000000 */

      /* 50 0x32 '2' */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0x06, /* 00000110 */
      0x1c, /* 00011100 */
      0x30, /* 00110000 */
      0x66, /* 01100110 */
      0xfe, /* 11111110 */
      0x00, /* 00000000 */

      /* 51 0x33 '3' */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0x06, /* 00000110 */
      0x3c, /* 00111100 */
      0x06, /* 00000110 */
      0xc6, /* 11000110 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 52 0x34 '4' */
      0x1c, /* 00011100 */
      0x3c, /* 00111100 */
      0x6c, /* 01101100 */
      0xcc, /* 11001100 */
      0xfe, /* 11111110 */
      0x0c, /* 00001100 */
      0x1e, /* 00011110 */
      0x00, /* 00000000 */

      /* 53 0x35 '5' */
      0xfe, /* 11111110 */
      0xc0, /* 11000000 */
      0xc0, /* 11000000 */
      0xfc, /* 11111100 */
      0x06, /* 00000110 */
      0xc6, /* 11000110 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 54 0x36 '6' */
      0x38, /* 00111000 */
      0x60, /* 01100000 */
      0xc0, /* 11000000 */
      0xfc, /* 11111100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 55 0x37 '7' */
      0xfe, /* 11111110 */
      0xc6, /* 11000110 */
      0x0c, /* 00001100 */
      0x18, /* 00011000 */
      0x30, /* 00110000 */
      0x30, /* 00110000 */
      0x30, /* 00110000 */
      0x00, /* 00000000 */

      /* 56 0x38 '8' */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 57 0x39 '9' */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x7e, /* 01111110 */
      0x06, /* 00000110 */
      0x0c, /* 00001100 */
      0x78, /* 01111000 */
      0x00, /* 00000000 */

      /* 58 0x3a ':' */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */

      /* 59 0x3b ';' */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x30, /* 00110000 */

      /* 60 0x3c ' <' */
      0x06, /* 00000110 */
      0x0c, /* 00001100 */
      0x18, /* 00011000 */
      0x30, /* 00110000 */
      0x18, /* 00011000 */
      0x0c, /* 00001100 */
      0x06, /* 00000110 */
      0x00, /* 00000000 */

      /* 61 0x3d '=' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7e, /* 01111110 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7e, /* 01111110 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 62 0x3e '>' */
      0x60, /* 01100000 */
      0x30, /* 00110000 */
      0x18, /* 00011000 */
      0x0c, /* 00001100 */
      0x18, /* 00011000 */
      0x30, /* 00110000 */
      0x60, /* 01100000 */
      0x00, /* 00000000 */

      /* 63 0x3f '?' */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0x0c, /* 00001100 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */

      /* 64 0x40 '@' */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xde, /* 11011110 */
      0xde, /* 11011110 */
      0xde, /* 11011110 */
      0xc0, /* 11000000 */
      0x78, /* 01111000 */
      0x00, /* 00000000 */

      /* 65 0x41 'A' */
      0x38, /* 00111000 */
      0x6c, /* 01101100 */
      0xc6, /* 11000110 */
      0xfe, /* 11111110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */

      /* 66 0x42 'B' */
      0xfc, /* 11111100 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x7c, /* 01111100 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0xfc, /* 11111100 */
      0x00, /* 00000000 */

      /* 67 0x43 'C' */
      0x3c, /* 00111100 */
      0x66, /* 01100110 */
      0xc0, /* 11000000 */
      0xc0, /* 11000000 */
      0xc0, /* 11000000 */
      0x66, /* 01100110 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */

      /* 68 0x44 'D' */
      0xf8, /* 11111000 */
      0x6c, /* 01101100 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x6c, /* 01101100 */
      0xf8, /* 11111000 */
      0x00, /* 00000000 */

      /* 69 0x45 'E' */
      0xfe, /* 11111110 */
      0x62, /* 01100010 */
      0x68, /* 01101000 */
      0x78, /* 01111000 */
      0x68, /* 01101000 */
      0x62, /* 01100010 */
      0xfe, /* 11111110 */
      0x00, /* 00000000 */

      /* 70 0x46 'F' */
      0xfe, /* 11111110 */
      0x62, /* 01100010 */
      0x68, /* 01101000 */
      0x78, /* 01111000 */
      0x68, /* 01101000 */
      0x60, /* 01100000 */
      0xf0, /* 11110000 */
      0x00, /* 00000000 */

      /* 71 0x47 'G' */
      0x3c, /* 00111100 */
      0x66, /* 01100110 */
      0xc0, /* 11000000 */
      0xc0, /* 11000000 */
      0xce, /* 11001110 */
      0x66, /* 01100110 */
      0x3a, /* 00111010 */
      0x00, /* 00000000 */

      /* 72 0x48 'H' */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xfe, /* 11111110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */

      /* 73 0x49 'I' */
      0x3c, /* 00111100 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */

      /* 74 0x4a 'J' */
      0x1e, /* 00011110 */
      0x0c, /* 00001100 */
      0x0c, /* 00001100 */
      0x0c, /* 00001100 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0x78, /* 01111000 */
      0x00, /* 00000000 */

      /* 75 0x4b 'K' */
      0xe6, /* 11100110 */
      0x66, /* 01100110 */
      0x6c, /* 01101100 */
      0x78, /* 01111000 */
      0x6c, /* 01101100 */
      0x66, /* 01100110 */
      0xe6, /* 11100110 */
      0x00, /* 00000000 */

      /* 76 0x4c 'L' */
      0xf0, /* 11110000 */
      0x60, /* 01100000 */
      0x60, /* 01100000 */
      0x60, /* 01100000 */
      0x62, /* 01100010 */
      0x66, /* 01100110 */
      0xfe, /* 11111110 */
      0x00, /* 00000000 */

      /* 77 0x4d 'M' */
      0xc6, /* 11000110 */
      0xee, /* 11101110 */
      0xfe, /* 11111110 */
      0xfe, /* 11111110 */
      0xd6, /* 11010110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */

      /* 78 0x4e 'N' */
      0xc6, /* 11000110 */
      0xe6, /* 11100110 */
      0xf6, /* 11110110 */
      0xde, /* 11011110 */
      0xce, /* 11001110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */

      /* 79 0x4f 'O' */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 80 0x50 'P' */
      0xfc, /* 11111100 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x7c, /* 01111100 */
      0x60, /* 01100000 */
      0x60, /* 01100000 */
      0xf0, /* 11110000 */
      0x00, /* 00000000 */

      /* 81 0x51 'Q' */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xce, /* 11001110 */
      0x7c, /* 01111100 */
      0x0e, /* 00001110 */

      /* 82 0x52 'R' */
      0xfc, /* 11111100 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x7c, /* 01111100 */
      0x6c, /* 01101100 */
      0x66, /* 01100110 */
      0xe6, /* 11100110 */
      0x00, /* 00000000 */

      /* 83 0x53 'S' */
      0x3c, /* 00111100 */
      0x66, /* 01100110 */
      0x30, /* 00110000 */
      0x18, /* 00011000 */
      0x0c, /* 00001100 */
      0x66, /* 01100110 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */

      /* 84 0x54 'T' */
      0x7e, /* 01111110 */
      0x7e, /* 01111110 */
      0x5a, /* 01011010 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */

      /* 85 0x55 'U' */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 86 0x56 'V' */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x6c, /* 01101100 */
      0x38, /* 00111000 */
      0x00, /* 00000000 */

      /* 87 0x57 'W' */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xd6, /* 11010110 */
      0xd6, /* 11010110 */
      0xfe, /* 11111110 */
      0x6c, /* 01101100 */
      0x00, /* 00000000 */

      /* 88 0x58 'X' */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x6c, /* 01101100 */
      0x38, /* 00111000 */
      0x6c, /* 01101100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */

      /* 89 0x59 'Y' */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x3c, /* 00111100 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */

      /* 90 0x5a 'Z' */
      0xfe, /* 11111110 */
      0xc6, /* 11000110 */
      0x8c, /* 10001100 */
      0x18, /* 00011000 */
      0x32, /* 00110010 */
      0x66, /* 01100110 */
      0xfe, /* 11111110 */
      0x00, /* 00000000 */

      /* 91 0x5b '[' */
      0x3c, /* 00111100 */
      0x30, /* 00110000 */
      0x30, /* 00110000 */
      0x30, /* 00110000 */
      0x30, /* 00110000 */
      0x30, /* 00110000 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */

      /* 92 0x5c '\' */
      0xc0, /* 11000000 */
      0x60, /* 01100000 */
      0x30, /* 00110000 */
      0x18, /* 00011000 */
      0x0c, /* 00001100 */
      0x06, /* 00000110 */
      0x02, /* 00000010 */
      0x00, /* 00000000 */

      /* 93 0x5d ']' */
      0x3c, /* 00111100 */
      0x0c, /* 00001100 */
      0x0c, /* 00001100 */
      0x0c, /* 00001100 */
      0x0c, /* 00001100 */
      0x0c, /* 00001100 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */

      /* 94 0x5e '^' */
      0x10, /* 00010000 */
      0x38, /* 00111000 */
      0x6c, /* 01101100 */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 95 0x5f '_' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xff, /* 11111111 */

      /* 96 0x60 '`' */
      0x30, /* 00110000 */
      0x18, /* 00011000 */
      0x0c, /* 00001100 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 97 0x61 'a' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x78, /* 01111000 */
      0x0c, /* 00001100 */
      0x7c, /* 01111100 */
      0xcc, /* 11001100 */
      0x76, /* 01110110 */
      0x00, /* 00000000 */

      /* 98 0x62 'b' */
      0xe0, /* 11100000 */
      0x60, /* 01100000 */
      0x7c, /* 01111100 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0xdc, /* 11011100 */
      0x00, /* 00000000 */

      /* 99 0x63 'c' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xc0, /* 11000000 */
      0xc6, /* 11000110 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 100 0x64 'd' */
      0x1c, /* 00011100 */
      0x0c, /* 00001100 */
      0x7c, /* 01111100 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0x76, /* 01110110 */
      0x00, /* 00000000 */

      /* 101 0x65 'e' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xfe, /* 11111110 */
      0xc0, /* 11000000 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 102 0x66 'f' */
      0x3c, /* 00111100 */
      0x66, /* 01100110 */
      0x60, /* 01100000 */
      0xf8, /* 11111000 */
      0x60, /* 01100000 */
      0x60, /* 01100000 */
      0xf0, /* 11110000 */
      0x00, /* 00000000 */

      /* 103 0x67 'g' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x76, /* 01110110 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0x7c, /* 01111100 */
      0x0c, /* 00001100 */
      0xf8, /* 11111000 */

      /* 104 0x68 'h' */
      0xe0, /* 11100000 */
      0x60, /* 01100000 */
      0x6c, /* 01101100 */
      0x76, /* 01110110 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0xe6, /* 11100110 */
      0x00, /* 00000000 */

      /* 105 0x69 'i' */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x38, /* 00111000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */

      /* 106 0x6a 'j' */
      0x06, /* 00000110 */
      0x00, /* 00000000 */
      0x06, /* 00000110 */
      0x06, /* 00000110 */
      0x06, /* 00000110 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x3c, /* 00111100 */

      /* 107 0x6b 'k' */
      0xe0, /* 11100000 */
      0x60, /* 01100000 */
      0x66, /* 01100110 */
      0x6c, /* 01101100 */
      0x78, /* 01111000 */
      0x6c, /* 01101100 */
      0xe6, /* 11100110 */
      0x00, /* 00000000 */

      /* 108 0x6c 'l' */
      0x38, /* 00111000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */

      /* 109 0x6d 'm' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xec, /* 11101100 */
      0xfe, /* 11111110 */
      0xd6, /* 11010110 */
      0xd6, /* 11010110 */
      0xd6, /* 11010110 */
      0x00, /* 00000000 */

      /* 110 0x6e 'n' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xdc, /* 11011100 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x00, /* 00000000 */

      /* 111 0x6f 'o' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 112 0x70 'p' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xdc, /* 11011100 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x7c, /* 01111100 */
      0x60, /* 01100000 */
      0xf0, /* 11110000 */

      /* 113 0x71 'q' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x76, /* 01110110 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0x7c, /* 01111100 */
      0x0c, /* 00001100 */
      0x1e, /* 00011110 */

      /* 114 0x72 'r' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xdc, /* 11011100 */
      0x76, /* 01110110 */
      0x60, /* 01100000 */
      0x60, /* 01100000 */
      0xf0, /* 11110000 */
      0x00, /* 00000000 */

      /* 115 0x73 's' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7e, /* 01111110 */
      0xc0, /* 11000000 */
      0x7c, /* 01111100 */
      0x06, /* 00000110 */
      0xfc, /* 11111100 */
      0x00, /* 00000000 */

      /* 116 0x74 't' */
      0x30, /* 00110000 */
      0x30, /* 00110000 */
      0xfc, /* 11111100 */
      0x30, /* 00110000 */
      0x30, /* 00110000 */
      0x36, /* 00110110 */
      0x1c, /* 00011100 */
      0x00, /* 00000000 */

      /* 117 0x75 'u' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0x76, /* 01110110 */
      0x00, /* 00000000 */

      /* 118 0x76 'v' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x6c, /* 01101100 */
      0x38, /* 00111000 */
      0x00, /* 00000000 */

      /* 119 0x77 'w' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xc6, /* 11000110 */
      0xd6, /* 11010110 */
      0xd6, /* 11010110 */
      0xfe, /* 11111110 */
      0x6c, /* 01101100 */
      0x00, /* 00000000 */

      /* 120 0x78 'x' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xc6, /* 11000110 */
      0x6c, /* 01101100 */
      0x38, /* 00111000 */
      0x6c, /* 01101100 */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */

      /* 121 0x79 'y' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x7e, /* 01111110 */
      0x06, /* 00000110 */
      0xfc, /* 11111100 */

      /* 122 0x7a 'z' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7e, /* 01111110 */
      0x4c, /* 01001100 */
      0x18, /* 00011000 */
      0x32, /* 00110010 */
      0x7e, /* 01111110 */
      0x00, /* 00000000 */

      /* 123 0x7b '{' */
      0x0e, /* 00001110 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x70, /* 01110000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x0e, /* 00001110 */
      0x00, /* 00000000 */

      /* 124 0x7c '|' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */

      /* 125 0x7d '}' */
      0x70, /* 01110000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x0e, /* 00001110 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x70, /* 01110000 */
      0x00, /* 00000000 */

      /* 126 0x7e '~' */
      0x76, /* 01110110 */
      0xdc, /* 11011100 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 127 0x7f '' */
      0x00, /* 00000000 */
      0x10, /* 00010000 */
      0x38, /* 00111000 */
      0x6c, /* 01101100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xfe, /* 11111110 */
      0x00, /* 00000000 */

      /* 128 0x80 '\u0080' */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xc0, /* 11000000 */
      0xc0, /* 11000000 */
      0xc6, /* 11000110 */
      0x7c, /* 01111100 */
      0x0c, /* 00001100 */
      0x78, /* 01111000 */

      /* 129 0x81 '\u0081' */
      0xcc, /* 11001100 */
      0x00, /* 00000000 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0x76, /* 01110110 */
      0x00, /* 00000000 */

      /* 130 0x82 '\u0082' */
      0x0c, /* 00001100 */
      0x18, /* 00011000 */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xfe, /* 11111110 */
      0xc0, /* 11000000 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 131 0x83 '\u0083' */
      0x7c, /* 01111100 */
      0x82, /* 10000010 */
      0x78, /* 01111000 */
      0x0c, /* 00001100 */
      0x7c, /* 01111100 */
      0xcc, /* 11001100 */
      0x76, /* 01110110 */
      0x00, /* 00000000 */

      /* 132 0x84 '\u0084' */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */
      0x78, /* 01111000 */
      0x0c, /* 00001100 */
      0x7c, /* 01111100 */
      0xcc, /* 11001100 */
      0x76, /* 01110110 */
      0x00, /* 00000000 */

      /* 133 0x85 '\u0085' */
      0x30, /* 00110000 */
      0x18, /* 00011000 */
      0x78, /* 01111000 */
      0x0c, /* 00001100 */
      0x7c, /* 01111100 */
      0xcc, /* 11001100 */
      0x76, /* 01110110 */
      0x00, /* 00000000 */

      /* 134 0x86 '\u0086' */
      0x30, /* 00110000 */
      0x30, /* 00110000 */
      0x78, /* 01111000 */
      0x0c, /* 00001100 */
      0x7c, /* 01111100 */
      0xcc, /* 11001100 */
      0x76, /* 01110110 */
      0x00, /* 00000000 */

      /* 135 0x87 '\u0087' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7e, /* 01111110 */
      0xc0, /* 11000000 */
      0xc0, /* 11000000 */
      0x7e, /* 01111110 */
      0x0c, /* 00001100 */
      0x38, /* 00111000 */

      /* 136 0x88 '\u0088' */
      0x7c, /* 01111100 */
      0x82, /* 10000010 */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xfe, /* 11111110 */
      0xc0, /* 11000000 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 137 0x89 '\u0089' */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xfe, /* 11111110 */
      0xc0, /* 11000000 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 138 0x8a '\u008a' */
      0x30, /* 00110000 */
      0x18, /* 00011000 */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xfe, /* 11111110 */
      0xc0, /* 11000000 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 139 0x8b '\u008b' */
      0x66, /* 01100110 */
      0x00, /* 00000000 */
      0x38, /* 00111000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */

      /* 140 0x8c '\u008c' */
      0x7c, /* 01111100 */
      0x82, /* 10000010 */
      0x38, /* 00111000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */

      /* 141 0x8d '\u008D' */
      0x30, /* 00110000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x38, /* 00111000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */

      /* 142 0x8e '\u008e' */
      0xc6, /* 11000110 */
      0x38, /* 00111000 */
      0x6c, /* 01101100 */
      0xc6, /* 11000110 */
      0xfe, /* 11111110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */

      /* 143 0x8f '\u008F' */
      0x38, /* 00111000 */
      0x6c, /* 01101100 */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xfe, /* 11111110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */

      /* 144 0x90 '\u0090' */
      0x18, /* 00011000 */
      0x30, /* 00110000 */
      0xfe, /* 11111110 */
      0xc0, /* 11000000 */
      0xf8, /* 11111000 */
      0xc0, /* 11000000 */
      0xfe, /* 11111110 */
      0x00, /* 00000000 */

      /* 145 0x91 '\u0091' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7e, /* 01111110 */
      0x18, /* 00011000 */
      0x7e, /* 01111110 */
      0xd8, /* 11011000 */
      0x7e, /* 01111110 */
      0x00, /* 00000000 */

      /* 146 0x92 '\u0092' */
      0x3e, /* 00111110 */
      0x6c, /* 01101100 */
      0xcc, /* 11001100 */
      0xfe, /* 11111110 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0xce, /* 11001110 */
      0x00, /* 00000000 */

      /* 147 0x93 '\u0093' */
      0x7c, /* 01111100 */
      0x82, /* 10000010 */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 148 0x94 '\u0094' */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 149 0x95 '\u0095' */
      0x30, /* 00110000 */
      0x18, /* 00011000 */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 150 0x96 '\u0096' */
      0x78, /* 01111000 */
      0x84, /* 10000100 */
      0x00, /* 00000000 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0x76, /* 01110110 */
      0x00, /* 00000000 */

      /* 151 0x97 '\u0097' */
      0x60, /* 01100000 */
      0x30, /* 00110000 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0x76, /* 01110110 */
      0x00, /* 00000000 */

      /* 152 0x98 '\u0098' */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x7e, /* 01111110 */
      0x06, /* 00000110 */
      0xfc, /* 11111100 */

      /* 153 0x99 '\u0099' */
      0xc6, /* 11000110 */
      0x38, /* 00111000 */
      0x6c, /* 01101100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x6c, /* 01101100 */
      0x38, /* 00111000 */
      0x00, /* 00000000 */

      /* 154 0x9a '\u009a' */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 155 0x9b '\u009b' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x7e, /* 01111110 */
      0xc0, /* 11000000 */
      0xc0, /* 11000000 */
      0x7e, /* 01111110 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 156 0x9c '\u009c' */
      0x38, /* 00111000 */
      0x6c, /* 01101100 */
      0x64, /* 01100100 */
      0xf0, /* 11110000 */
      0x60, /* 01100000 */
      0x66, /* 01100110 */
      0xfc, /* 11111100 */
      0x00, /* 00000000 */

      /* 157 0x9d '\u009d' */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x3c, /* 00111100 */
      0x7e, /* 01111110 */
      0x18, /* 00011000 */
      0x7e, /* 01111110 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 158 0x9e '\u009e' */
      0xf8, /* 11111000 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0xfa, /* 11111010 */
      0xc6, /* 11000110 */
      0xcf, /* 11001111 */
      0xc6, /* 11000110 */
      0xc7, /* 11000111 */

      /* 159 0x9f '\u009f' */
      0x0e, /* 00001110 */
      0x1b, /* 00011011 */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x18, /* 00011000 */
      0xd8, /* 11011000 */
      0x70, /* 01110000 */
      0x00, /* 00000000 */

      /* 160 0xa0 '\u00a0' */
      0x18, /* 00011000 */
      0x30, /* 00110000 */
      0x78, /* 01111000 */
      0x0c, /* 00001100 */
      0x7c, /* 01111100 */
      0xcc, /* 11001100 */
      0x76, /* 01110110 */
      0x00, /* 00000000 */

      /* 161 0xa1 '\u00a1' */
      0x0c, /* 00001100 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x38, /* 00111000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */

      /* 162 0xa2 '\u00a2' */
      0x0c, /* 00001100 */
      0x18, /* 00011000 */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */

      /* 163 0xa3 '\u00a3' */
      0x18, /* 00011000 */
      0x30, /* 00110000 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0x76, /* 01110110 */
      0x00, /* 00000000 */

      /* 164 0xa4 '\u00a4' */
      0x76, /* 01110110 */
      0xdc, /* 11011100 */
      0x00, /* 00000000 */
      0xdc, /* 11011100 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x00, /* 00000000 */

      /* 165 0xa5 '\u00a5' */
      0x76, /* 01110110 */
      0xdc, /* 11011100 */
      0x00, /* 00000000 */
      0xe6, /* 11100110 */
      0xf6, /* 11110110 */
      0xde, /* 11011110 */
      0xce, /* 11001110 */
      0x00, /* 00000000 */

      /* 166 0xa6 '\u00a6' */
      0x3c, /* 00111100 */
      0x6c, /* 01101100 */
      0x6c, /* 01101100 */
      0x3e, /* 00111110 */
      0x00, /* 00000000 */
      0x7e, /* 01111110 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 167 0xa7 '\u00a7' */
      0x38, /* 00111000 */
      0x6c, /* 01101100 */
      0x6c, /* 01101100 */
      0x38, /* 00111000 */
      0x00, /* 00000000 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 168 0xa8 '\u00a8' */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x30, /* 00110000 */
      0x63, /* 01100011 */
      0x3e, /* 00111110 */
      0x00, /* 00000000 */

      /* 169 0xa9 '\u00a9' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xfe, /* 11111110 */
      0xc0, /* 11000000 */
      0xc0, /* 11000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 170 0xaa '\u00aa' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xfe, /* 11111110 */
      0x06, /* 00000110 */
      0x06, /* 00000110 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 171 0xab '\u00ab' */
      0x63, /* 01100011 */
      0xe6, /* 11100110 */
      0x6c, /* 01101100 */
      0x7e, /* 01111110 */
      0x33, /* 00110011 */
      0x66, /* 01100110 */
      0xcc, /* 11001100 */
      0x0f, /* 00001111 */

      /* 172 0xac '\u00ac' */
      0x63, /* 01100011 */
      0xe6, /* 11100110 */
      0x6c, /* 01101100 */
      0x7a, /* 01111010 */
      0x36, /* 00110110 */
      0x6a, /* 01101010 */
      0xdf, /* 11011111 */
      0x06, /* 00000110 */

      /* 173 0xad '\u00ad' */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x3c, /* 00111100 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */

      /* 174 0xae '\u00ae' */
      0x00, /* 00000000 */
      0x33, /* 00110011 */
      0x66, /* 01100110 */
      0xcc, /* 11001100 */
      0x66, /* 01100110 */
      0x33, /* 00110011 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 175 0xaf '\u00af' */
      0x00, /* 00000000 */
      0xcc, /* 11001100 */
      0x66, /* 01100110 */
      0x33, /* 00110011 */
      0x66, /* 01100110 */
      0xcc, /* 11001100 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 176 0xb0 '\u00b0' */
      0x22, /* 00100010 */
      0x88, /* 10001000 */
      0x22, /* 00100010 */
      0x88, /* 10001000 */
      0x22, /* 00100010 */
      0x88, /* 10001000 */
      0x22, /* 00100010 */
      0x88, /* 10001000 */

      /* 177 0xb1 '\u00b1' */
      0x55, /* 01010101 */
      0xaa, /* 10101010 */
      0x55, /* 01010101 */
      0xaa, /* 10101010 */
      0x55, /* 01010101 */
      0xaa, /* 10101010 */
      0x55, /* 01010101 */
      0xaa, /* 10101010 */

      /* 178 0xb2 '\u00b2' */
      0x77, /* 01110111 */
      0xdd, /* 11011101 */
      0x77, /* 01110111 */
      0xdd, /* 11011101 */
      0x77, /* 01110111 */
      0xdd, /* 11011101 */
      0x77, /* 01110111 */
      0xdd, /* 11011101 */

      /* 179 0xb3 '\u00b3' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 180 0xb4 '\u00b4' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0xf8, /* 11111000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 181 0xb5 '\u00b5' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0xf8, /* 11111000 */
      0x18, /* 00011000 */
      0xf8, /* 11111000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 182 0xb6 '\u00b6' */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0xf6, /* 11110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */

      /* 183 0xb7 '\u00b7' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xfe, /* 11111110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */

      /* 184 0xb8 '\u00b8' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xf8, /* 11111000 */
      0x18, /* 00011000 */
      0xf8, /* 11111000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 185 0xb9 '\u00b9' */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0xf6, /* 11110110 */
      0x06, /* 00000110 */
      0xf6, /* 11110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */

      /* 186 0xba '\u00ba' */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */

      /* 187 0xbb '\u00bb' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xfe, /* 11111110 */
      0x06, /* 00000110 */
      0xf6, /* 11110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */

      /* 188 0xbc '\u00bc' */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0xf6, /* 11110110 */
      0x06, /* 00000110 */
      0xfe, /* 11111110 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 189 0xbd '\u00bd' */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0xfe, /* 11111110 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 190 0xbe '\u00be' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0xf8, /* 11111000 */
      0x18, /* 00011000 */
      0xf8, /* 11111000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 191 0xbf '\u00bf' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xf8, /* 11111000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 192 0xc0 '\u00c0' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x1f, /* 00011111 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 193 0xc1 '\u00c1' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0xff, /* 11111111 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 194 0xc2 '\u00c2' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xff, /* 11111111 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 195 0xc3 '\u00c3' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x1f, /* 00011111 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 196 0xc4 '\u00c4' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xff, /* 11111111 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 197 0xc5 '\u00c5' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0xff, /* 11111111 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 198 0xc6 '\u00c6' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x1f, /* 00011111 */
      0x18, /* 00011000 */
      0x1f, /* 00011111 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 199 0xc7 '\u00c7' */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x37, /* 00110111 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */

      /* 200 0xc8 '\u00c8' */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x37, /* 00110111 */
      0x30, /* 00110000 */
      0x3f, /* 00111111 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 201 0xc9 '\u00c9' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x3f, /* 00111111 */
      0x30, /* 00110000 */
      0x37, /* 00110111 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */

      /* 202 0xca '\u00ca' */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0xf7, /* 11110111 */
      0x00, /* 00000000 */
      0xff, /* 11111111 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 203 0xcb '\u00cb' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xff, /* 11111111 */
      0x00, /* 00000000 */
      0xf7, /* 11110111 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */

      /* 204 0xcc '\u00cc' */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x37, /* 00110111 */
      0x30, /* 00110000 */
      0x37, /* 00110111 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */

      /* 205 0xcd '\u00cd' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xff, /* 11111111 */
      0x00, /* 00000000 */
      0xff, /* 11111111 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 206 0xce '\u00ce' */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0xf7, /* 11110111 */
      0x00, /* 00000000 */
      0xf7, /* 11110111 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */

      /* 207 0xcf '\u00cf' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0xff, /* 11111111 */
      0x00, /* 00000000 */
      0xff, /* 11111111 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 208 0xd0 '\u00d0' */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0xff, /* 11111111 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 209 0xd1 '\u00d1' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xff, /* 11111111 */
      0x00, /* 00000000 */
      0xff, /* 11111111 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 210 0xd2 '\u00d2' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xff, /* 11111111 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */

      /* 211 0xd3 '\u00d3' */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x3f, /* 00111111 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 212 0xd4 '\u00d4' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x1f, /* 00011111 */
      0x18, /* 00011000 */
      0x1f, /* 00011111 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 213 0xd5 '\u00d5' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x1f, /* 00011111 */
      0x18, /* 00011000 */
      0x1f, /* 00011111 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 214 0xd6 '\u00d6' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x3f, /* 00111111 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */

      /* 215 0xd7 '\u00d7' */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0xff, /* 11111111 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */

      /* 216 0xd8 '\u00d8' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0xff, /* 11111111 */
      0x18, /* 00011000 */
      0xff, /* 11111111 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 217 0xd9 '\u00d9' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0xf8, /* 11111000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 218 0xda '\u00da' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x1f, /* 00011111 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 219 0xdb '\u00db' */
      0xff, /* 11111111 */
      0xff, /* 11111111 */
      0xff, /* 11111111 */
      0xff, /* 11111111 */
      0xff, /* 11111111 */
      0xff, /* 11111111 */
      0xff, /* 11111111 */
      0xff, /* 11111111 */

      /* 220 0xdc '\u00dc' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xff, /* 11111111 */
      0xff, /* 11111111 */
      0xff, /* 11111111 */
      0xff, /* 11111111 */

      /* 221 0xdd '\u00dd' */
      0xf0, /* 11110000 */
      0xf0, /* 11110000 */
      0xf0, /* 11110000 */
      0xf0, /* 11110000 */
      0xf0, /* 11110000 */
      0xf0, /* 11110000 */
      0xf0, /* 11110000 */
      0xf0, /* 11110000 */

      /* 222 0xde '\u00de' */
      0x0f, /* 00001111 */
      0x0f, /* 00001111 */
      0x0f, /* 00001111 */
      0x0f, /* 00001111 */
      0x0f, /* 00001111 */
      0x0f, /* 00001111 */
      0x0f, /* 00001111 */
      0x0f, /* 00001111 */

      /* 223 0xdf '\u00df' */
      0xff, /* 11111111 */
      0xff, /* 11111111 */
      0xff, /* 11111111 */
      0xff, /* 11111111 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 224 0xe0 '\u00e0' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x76, /* 01110110 */
      0xdc, /* 11011100 */
      0xc8, /* 11001000 */
      0xdc, /* 11011100 */
      0x76, /* 01110110 */
      0x00, /* 00000000 */

      /* 225 0xe1 '\u00e1' */
      0x78, /* 01111000 */
      0xcc, /* 11001100 */
      0xcc, /* 11001100 */
      0xd8, /* 11011000 */
      0xcc, /* 11001100 */
      0xc6, /* 11000110 */
      0xcc, /* 11001100 */
      0x00, /* 00000000 */

      /* 226 0xe2 '\u00e2' */
      0xfe, /* 11111110 */
      0xc6, /* 11000110 */
      0xc0, /* 11000000 */
      0xc0, /* 11000000 */
      0xc0, /* 11000000 */
      0xc0, /* 11000000 */
      0xc0, /* 11000000 */
      0x00, /* 00000000 */

      /* 227 0xe3 '\u00e3' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0xfe, /* 11111110 */
      0x6c, /* 01101100 */
      0x6c, /* 01101100 */
      0x6c, /* 01101100 */
      0x6c, /* 01101100 */
      0x00, /* 00000000 */

      /* 228 0xe4 '\u00e4' */
      0xfe, /* 11111110 */
      0xc6, /* 11000110 */
      0x60, /* 01100000 */
      0x30, /* 00110000 */
      0x60, /* 01100000 */
      0xc6, /* 11000110 */
      0xfe, /* 11111110 */
      0x00, /* 00000000 */

      /* 229 0xe5 '\u00e5' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7e, /* 01111110 */
      0xd8, /* 11011000 */
      0xd8, /* 11011000 */
      0xd8, /* 11011000 */
      0x70, /* 01110000 */
      0x00, /* 00000000 */

      /* 230 0xe6 '\u00e6' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x7c, /* 01111100 */
      0xc0, /* 11000000 */

      /* 231 0xe7 '\u00e7' */
      0x00, /* 00000000 */
      0x76, /* 01110110 */
      0xdc, /* 11011100 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */

      /* 232 0xe8 '\u00e8' */
      0x7e, /* 01111110 */
      0x18, /* 00011000 */
      0x3c, /* 00111100 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x3c, /* 00111100 */
      0x18, /* 00011000 */
      0x7e, /* 01111110 */

      /* 233 0xe9 '\u00e9' */
      0x38, /* 00111000 */
      0x6c, /* 01101100 */
      0xc6, /* 11000110 */
      0xfe, /* 11111110 */
      0xc6, /* 11000110 */
      0x6c, /* 01101100 */
      0x38, /* 00111000 */
      0x00, /* 00000000 */

      /* 234 0xea '\u00ea' */
      0x38, /* 00111000 */
      0x6c, /* 01101100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x6c, /* 01101100 */
      0x6c, /* 01101100 */
      0xee, /* 11101110 */
      0x00, /* 00000000 */

      /* 235 0xeb '\u00eb' */
      0x0e, /* 00001110 */
      0x18, /* 00011000 */
      0x0c, /* 00001100 */
      0x3e, /* 00111110 */
      0x66, /* 01100110 */
      0x66, /* 01100110 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */

      /* 236 0xec '\u00ec' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7e, /* 01111110 */
      0xdb, /* 11011011 */
      0xdb, /* 11011011 */
      0x7e, /* 01111110 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 237 0xed '\u00ed' */
      0x06, /* 00000110 */
      0x0c, /* 00001100 */
      0x7e, /* 01111110 */
      0xdb, /* 11011011 */
      0xdb, /* 11011011 */
      0x7e, /* 01111110 */
      0x60, /* 01100000 */
      0xc0, /* 11000000 */

      /* 238 0xee '\u00ee' */
      0x1e, /* 00011110 */
      0x30, /* 00110000 */
      0x60, /* 01100000 */
      0x7e, /* 01111110 */
      0x60, /* 01100000 */
      0x30, /* 00110000 */
      0x1e, /* 00011110 */
      0x00, /* 00000000 */

      /* 239 0xef '\u00ef' */
      0x00, /* 00000000 */
      0x7c, /* 01111100 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0xc6, /* 11000110 */
      0x00, /* 00000000 */

      /* 240 0xf0 '\u00f0' */
      0x00, /* 00000000 */
      0xfe, /* 11111110 */
      0x00, /* 00000000 */
      0xfe, /* 11111110 */
      0x00, /* 00000000 */
      0xfe, /* 11111110 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 241 0xf1 '\u00f1' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x7e, /* 01111110 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x7e, /* 01111110 */
      0x00, /* 00000000 */

      /* 242 0xf2 '\u00f2' */
      0x30, /* 00110000 */
      0x18, /* 00011000 */
      0x0c, /* 00001100 */
      0x18, /* 00011000 */
      0x30, /* 00110000 */
      0x00, /* 00000000 */
      0x7e, /* 01111110 */
      0x00, /* 00000000 */

      /* 243 0xf3 '\u00f3' */
      0x0c, /* 00001100 */
      0x18, /* 00011000 */
      0x30, /* 00110000 */
      0x18, /* 00011000 */
      0x0c, /* 00001100 */
      0x00, /* 00000000 */
      0x7e, /* 01111110 */
      0x00, /* 00000000 */

      /* 244 0xf4 '\u00f4' */
      0x0e, /* 00001110 */
      0x1b, /* 00011011 */
      0x1b, /* 00011011 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */

      /* 245 0xf5 '\u00f5' */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0xd8, /* 11011000 */
      0xd8, /* 11011000 */
      0x70, /* 01110000 */

      /* 246 0xf6 '\u00f6' */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x7e, /* 01111110 */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 247 0xf7 '\u00f7' */
      0x00, /* 00000000 */
      0x76, /* 01110110 */
      0xdc, /* 11011100 */
      0x00, /* 00000000 */
      0x76, /* 01110110 */
      0xdc, /* 11011100 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 248 0xf8 '\u00f8' */
      0x38, /* 00111000 */
      0x6c, /* 01101100 */
      0x6c, /* 01101100 */
      0x38, /* 00111000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 249 0xf9 '\u00f9' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 250 0xfa '\u00fa' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x18, /* 00011000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 251 0xfb '\u00fb' */
      0x0f, /* 00001111 */
      0x0c, /* 00001100 */
      0x0c, /* 00001100 */
      0x0c, /* 00001100 */
      0xec, /* 11101100 */
      0x6c, /* 01101100 */
      0x3c, /* 00111100 */
      0x1c, /* 00011100 */

      /* 252 0xfc '\u00fc' */
      0x6c, /* 01101100 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x36, /* 00110110 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 253 0xfd '\u00fd' */
      0x78, /* 01111000 */
      0x0c, /* 00001100 */
      0x18, /* 00011000 */
      0x30, /* 00110000 */
      0x7c, /* 01111100 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 254 0xfe '\u00fe' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x3c, /* 00111100 */
      0x3c, /* 00111100 */
      0x3c, /* 00111100 */
      0x3c, /* 00111100 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 255 0xff '\u00ff' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00
   /* 00000000 */
   };

   private static final int[] font_6x11 =
   {
      /* 0 0x00 '^A' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 1 0x01 '^B' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 2 0x02 '^C' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 3 0x03 '^D' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 4 0x04 '^E' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 5 0x05 '^F' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 6 0x06 '^G' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 7 0x07 '^H' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 8 0x08 '^I' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 9 0x09 '^J' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 10 0x0a '^K' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 11 0x0b '^L' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 12 0x0c '^M' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 13 0x0d '^N' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 14 0x0e '^O' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 15 0x0f '^P' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 16 0x10 '^Q' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 17 0x11 '^R' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x28, /* 00 0 000 */
      0x54, /* 0 0 0 00 */
      0x38, /* 00 000 */
      0x54, /* 0 0 0 00 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 18 0x12 '^S' */
      0x04, /* 00000 00 */
      0x04, /* 00000 00 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x50, /* 0 0 0000 */
      0x50, /* 0 0 0000 */
      0x20, /* 00 00000 */
      0x20, /* 00 00000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 19 0x13 '^T' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x38, /* 00 000 */
      0x7c, /* 0 00 */
      0x38, /* 00 000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 20 0x14 '^U' */
      0x18, /* 000 000 */
      0x10, /* 000 0000 */
      0x28, /* 00 0 000 */
      0x7c, /* 0 00 */
      0x78, /* 0 000 */
      0x78, /* 0 000 */
      0x7c, /* 0 00 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 21 0x15 '^V' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 22 0x16 '^W' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 23 0x17 '^X' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 24 0x18 '^Y' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 25 0x19 '^Z' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 26 0x1a '^[' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 27 0x1b '^\' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 28 0x1c '^]' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 29 0x1d '^^' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 30 0x1e '^_' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 31 0x1f '^`' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 32 0x20 ' ' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 33 0x21 '!' */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 34 0x22 '"' */
      0x28, /* 00 0 000 */
      0x28, /* 00 0 000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 35 0x23 '#' */
      0x00, /* 00000000 */
      0x28, /* 00 0 000 */
      0x7c, /* 0 00 */
      0x28, /* 00 0 000 */
      0x7c, /* 0 00 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 36 0x24 '$' */
      0x10, /* 000 0000 */
      0x38, /* 00 000 */
      0x54, /* 0 0 0 00 */
      0x50, /* 0 0 0000 */
      0x38, /* 00 000 */
      0x14, /* 000 0 00 */
      0x54, /* 0 0 0 00 */
      0x38, /* 00 000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 37 0x25 '%' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x54, /* 0 0 0 00 */
      0x58, /* 0 0 000 */
      0x28, /* 00 0 000 */
      0x34, /* 00 0 00 */
      0x54, /* 0 0 0 00 */
      0x48, /* 0 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 38 0x26 '&' */
      0x00, /* 00000000 */
      0x30, /* 00 0000 */
      0x48, /* 0 00 000 */
      0x50, /* 0 0 0000 */
      0x20, /* 00 00000 */
      0x54, /* 0 0 0 00 */
      0x48, /* 0 00 000 */
      0x34, /* 00 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 39 0x27 ''' */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 40 0x28 '(' */
      0x04, /* 00000 00 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x08, /* 0000 000 */
      0x04, /* 00000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 41 0x29 ')' */
      0x20, /* 00 00000 */
      0x10, /* 000 0000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x20, /* 00 00000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 42 0x2a '*' */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x54, /* 0 0 0 00 */
      0x38, /* 00 000 */
      0x54, /* 0 0 0 00 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 43 0x2b '+' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x7c, /* 0 00 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 44 0x2c ',' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x30, /* 00 0000 */
      0x30, /* 00 0000 */
      0x10, /* 000 0000 */
      0x20, /* 00 00000 */
      0x00, /* 00000000 */

      /* 45 0x2d '-' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7c, /* 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 46 0x2e '.' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x18, /* 000 000 */
      0x18, /* 000 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 47 0x2f '/' */
      0x04, /* 00000 00 */
      0x04, /* 00000 00 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x20, /* 00 00000 */
      0x20, /* 00 00000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x00, /* 00000000 */

      /* 48 0x30 '0' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x4c, /* 0 00 00 */
      0x54, /* 0 0 0 00 */
      0x64, /* 0 00 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 49 0x31 '1' */
      0x00, /* 00000000 */
      0x08, /* 0000 000 */
      0x18, /* 000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x1c, /* 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 50 0x32 '2' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x04, /* 00000 00 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x20, /* 00 00000 */
      0x7c, /* 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 51 0x33 '3' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x04, /* 00000 00 */
      0x18, /* 000 000 */
      0x04, /* 00000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 52 0x34 '4' */
      0x00, /* 00000000 */
      0x08, /* 0000 000 */
      0x18, /* 000 000 */
      0x28, /* 00 0 000 */
      0x48, /* 0 00 000 */
      0x7c, /* 0 00 */
      0x08, /* 0000 000 */
      0x1c, /* 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 53 0x35 '5' */
      0x00, /* 00000000 */
      0x7c, /* 0 00 */
      0x40, /* 0 000000 */
      0x78, /* 0 000 */
      0x04, /* 00000 00 */
      0x04, /* 00000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 54 0x36 '6' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x40, /* 0 000000 */
      0x78, /* 0 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 55 0x37 '7' */
      0x00, /* 00000000 */
      0x7c, /* 0 00 */
      0x04, /* 00000 00 */
      0x04, /* 00000 00 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 56 0x38 '8' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 57 0x39 '9' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x3c, /* 00 00 */
      0x04, /* 00000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 58 0x3a ':' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x18, /* 000 000 */
      0x18, /* 000 000 */
      0x00, /* 00000000 */
      0x18, /* 000 000 */
      0x18, /* 000 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 59 0x3b ';' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x30, /* 00 0000 */
      0x30, /* 00 0000 */
      0x00, /* 00000000 */
      0x30, /* 00 0000 */
      0x30, /* 00 0000 */
      0x10, /* 000 0000 */
      0x20, /* 00 00000 */
      0x00, /* 00000000 */

      /* 60 0x3c ' <' */
      0x00, /* 00000000 */
      0x04, /* 00000 00 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x20, /* 00 00000 */
      0x10, /* 000 0000 */
      0x08, /* 0000 000 */
      0x04, /* 00000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 61 0x3d '=' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7c, /* 0 00 */
      0x00, /* 00000000 */
      0x7c, /* 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 62 0x3e '>' */
      0x00, /* 00000000 */
      0x20, /* 00 00000 */
      0x10, /* 000 0000 */
      0x08, /* 0000 000 */
      0x04, /* 00000 00 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x20, /* 00 00000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 63 0x3f '?' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x04, /* 00000 00 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 64 0x40 '@' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x74, /* 0 0 00 */
      0x54, /* 0 0 0 00 */
      0x78, /* 0 000 */
      0x40, /* 0 000000 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 65 0x41 'A' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x7c, /* 0 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 66 0x42 'B' */
      0x00, /* 00000000 */
      0x78, /* 0 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x78, /* 0 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x78, /* 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 67 0x43 'C' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 68 0x44 'D' */
      0x00, /* 00000000 */
      0x78, /* 0 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x78, /* 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 69 0x45 'E' */
      0x00, /* 00000000 */
      0x7c, /* 0 00 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x78, /* 0 000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x7c, /* 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 70 0x46 'F' */
      0x00, /* 00000000 */
      0x7c, /* 0 00 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x78, /* 0 000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 71 0x47 'G' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x40, /* 0 000000 */
      0x4c, /* 0 00 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 72 0x48 'H' */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x7c, /* 0 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 73 0x49 'I' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 74 0x4a 'J' */
      0x00, /* 00000000 */
      0x04, /* 00000 00 */
      0x04, /* 00000 00 */
      0x04, /* 00000 00 */
      0x04, /* 00000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 75 0x4b 'K' */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x48, /* 0 00 000 */
      0x50, /* 0 0 0000 */
      0x60, /* 0 00000 */
      0x50, /* 0 0 0000 */
      0x48, /* 0 00 000 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 76 0x4c 'L' */
      0x00, /* 00000000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x7c, /* 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 77 0x4d 'M' */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x6c, /* 0 0 00 */
      0x54, /* 0 0 0 00 */
      0x54, /* 0 0 0 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 78 0x4e 'N' */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x64, /* 0 00 00 */
      0x54, /* 0 0 0 00 */
      0x4c, /* 0 00 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 79 0x4f 'O' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 80 0x50 'P' */
      0x00, /* 00000000 */
      0x78, /* 0 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x78, /* 0 000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 81 0x51 'Q' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x54, /* 0 0 0 00 */
      0x38, /* 00 000 */
      0x04, /* 00000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 82 0x52 'R' */
      0x00, /* 00000000 */
      0x78, /* 0 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x78, /* 0 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 83 0x53 'S' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x40, /* 0 000000 */
      0x38, /* 00 000 */
      0x04, /* 00000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 84 0x54 'T' */
      0x00, /* 00000000 */
      0x7c, /* 0 00 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 85 0x55 'U' */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 86 0x56 'V' */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x28, /* 00 0 000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 87 0x57 'W' */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x54, /* 0 0 0 00 */
      0x54, /* 0 0 0 00 */
      0x6c, /* 0 0 00 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 88 0x58 'X' */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x28, /* 00 0 000 */
      0x10, /* 000 0000 */
      0x28, /* 00 0 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 89 0x59 'Y' */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x28, /* 00 0 000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 90 0x5a 'Z' */
      0x00, /* 00000000 */
      0x7c, /* 0 00 */
      0x04, /* 00000 00 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x20, /* 00 00000 */
      0x40, /* 0 000000 */
      0x7c, /* 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 91 0x5b '[' */
      0x0c, /* 0000 00 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x0c, /* 0000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 92 0x5c '\' */
      0x20, /* 00 00000 */
      0x20, /* 00 00000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x04, /* 00000 00 */
      0x04, /* 00000 00 */
      0x02, /* 000000 0 */
      0x02, /* 000000 0 */
      0x00, /* 00000000 */

      /* 93 0x5d ']' */
      0x30, /* 00 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x30, /* 00 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 94 0x5e '^' */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x28, /* 00 0 000 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 95 0x5f '_' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7e, /* 0 0 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 96 0x60 '`' */
      0x20, /* 00 00000 */
      0x10, /* 000 0000 */
      0x08, /* 0000 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 97 0x61 'a' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x4c, /* 0 00 00 */
      0x34, /* 00 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 98 0x62 'b' */
      0x00, /* 00000000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x78, /* 0 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x78, /* 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 99 0x63 'c' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x40, /* 0 000000 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 100 0x64 'd' */
      0x00, /* 00000000 */
      0x04, /* 00000 00 */
      0x04, /* 00000 00 */
      0x3c, /* 00 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 101 0x65 'e' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x7c, /* 0 00 */
      0x40, /* 0 000000 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 102 0x66 'f' */
      0x00, /* 00000000 */
      0x0c, /* 0000 00 */
      0x10, /* 000 0000 */
      0x38, /* 00 000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 103 0x67 'g' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x3c, /* 00 00 */
      0x04, /* 00000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */

      /* 104 0x68 'h' */
      0x00, /* 00000000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x78, /* 0 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 105 0x69 'i' */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x30, /* 00 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 106 0x6a 'j' */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x30, /* 00 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x60, /* 0 00000 */
      0x00, /* 00000000 */

      /* 107 0x6b 'k' */
      0x00, /* 00000000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x48, /* 0 00 000 */
      0x50, /* 0 0 0000 */
      0x70, /* 0 0000 */
      0x48, /* 0 00 000 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 108 0x6c 'l' */
      0x00, /* 00000000 */
      0x30, /* 00 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 109 0x6d 'm' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x78, /* 0 000 */
      0x54, /* 0 0 0 00 */
      0x54, /* 0 0 0 00 */
      0x54, /* 0 0 0 00 */
      0x54, /* 0 0 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 110 0x6e 'n' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x58, /* 0 0 000 */
      0x64, /* 0 00 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 111 0x6f 'o' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 112 0x70 'p' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x78, /* 0 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x78, /* 0 000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x00, /* 00000000 */

      /* 113 0x71 'q' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x3c, /* 00 00 */
      0x04, /* 00000 00 */
      0x04, /* 00000 00 */
      0x00, /* 00000000 */

      /* 114 0x72 'r' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x58, /* 0 0 000 */
      0x64, /* 0 00 00 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 115 0x73 's' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x40, /* 0 000000 */
      0x38, /* 00 000 */
      0x04, /* 00000 00 */
      0x78, /* 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 116 0x74 't' */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x38, /* 00 000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x0c, /* 0000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 117 0x75 'u' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x4c, /* 0 00 00 */
      0x34, /* 00 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 118 0x76 'v' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x28, /* 00 0 000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 119 0x77 'w' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x54, /* 0 0 0 00 */
      0x54, /* 0 0 0 00 */
      0x54, /* 0 0 0 00 */
      0x54, /* 0 0 0 00 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 120 0x78 'x' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x28, /* 00 0 000 */
      0x10, /* 000 0000 */
      0x28, /* 00 0 000 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 121 0x79 'y' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x3c, /* 00 00 */
      0x04, /* 00000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */

      /* 122 0x7a 'z' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7c, /* 0 00 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x20, /* 00 00000 */
      0x7c, /* 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 123 0x7b '{' */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x04, /* 00000 00 */
      0x00, /* 00000000 */

      /* 124 0x7c '|' */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 125 0x7d '}' */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x20, /* 00 00000 */
      0x00, /* 00000000 */

      /* 126 0x7e '~' */
      0x00, /* 00000000 */
      0x34, /* 00 0 00 */
      0x58, /* 0 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 127 0x7f '^?' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 128 0x80 '\200' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x7c, /* 0 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 129 0x81 '\201' */
      0x28, /* 00 0 000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x7c, /* 0 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 130 0x82 '\202' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 131 0x83 '\203' */
      0x10, /* 000 0000 */
      0x7c, /* 0 00 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x78, /* 0 000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x7c, /* 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 132 0x84 '\204' */
      0x58, /* 0 0 000 */
      0x44, /* 0 000 00 */
      0x64, /* 0 00 00 */
      0x54, /* 0 0 0 00 */
      0x4c, /* 0 00 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 133 0x85 '\205' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 134 0x86 '\206' */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 135 0x87 '\207' */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x4c, /* 0 00 00 */
      0x34, /* 00 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 136 0x88 '\210' */
      0x10, /* 000 0000 */
      0x08, /* 0000 000 */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x4c, /* 0 00 00 */
      0x34, /* 00 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 137 0x89 '\211' */
      0x10, /* 000 0000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x4c, /* 0 00 00 */
      0x34, /* 00 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 138 0x8a '\212' */
      0x00, /* 00000000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x4c, /* 0 00 00 */
      0x34, /* 00 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 139 0x8b '\213' */
      0x34, /* 00 0 00 */
      0x58, /* 0 0 000 */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x4c, /* 0 00 00 */
      0x34, /* 00 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 140 0x8c '\214' */
      0x18, /* 000 000 */
      0x24, /* 00 00 00 */
      0x18, /* 000 000 */
      0x3c, /* 00 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x4c, /* 0 00 00 */
      0x34, /* 00 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 141 0x8d '\215' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x3c, /* 00 00 */
      0x10, /* 000 0000 */
      0x20, /* 00 00000 */
      0x00, /* 00000000 */

      /* 142 0x8e '\216' */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x7c, /* 0 00 */
      0x40, /* 0 000000 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 143 0x8f '\217' */
      0x20, /* 00 00000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x7c, /* 0 00 */
      0x40, /* 0 000000 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 144 0x90 '\220' */
      0x10, /* 000 0000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x7c, /* 0 00 */
      0x40, /* 0 000000 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 145 0x91 '\221' */
      0x00, /* 00000000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x7c, /* 0 00 */
      0x40, /* 0 000000 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 146 0x92 '\222' */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 147 0x93 '\223' */
      0x20, /* 00 00000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 148 0x94 '\224' */
      0x10, /* 000 0000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 149 0x95 '\225' */
      0x00, /* 00000000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 150 0x96 '\226' */
      0x34, /* 00 0 00 */
      0x58, /* 0 0 000 */
      0x00, /* 00000000 */
      0x58, /* 0 0 000 */
      0x64, /* 0 00 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 151 0x97 '\227' */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 152 0x98 '\230' */
      0x20, /* 00 00000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 153 0x99 '\231' */
      0x10, /* 000 0000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 154 0x9a '\232' */
      0x00, /* 00000000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 155 0x9b '\233' */
      0x34, /* 00 0 00 */
      0x58, /* 0 0 000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 156 0x9c '\234' */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x4c, /* 0 00 00 */
      0x34, /* 00 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 157 0x9d '\235' */
      0x20, /* 00 00000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x4c, /* 0 00 00 */
      0x34, /* 00 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 158 0x9e '\236' */
      0x10, /* 000 0000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x4c, /* 0 00 00 */
      0x34, /* 00 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 159 0x9f '\237' */
      0x00, /* 00000000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x4c, /* 0 00 00 */
      0x34, /* 00 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 160 0xa0 '\240' */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x38, /* 00 000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 161 0xa1 '\241' */
      0x18, /* 000 000 */
      0x24, /* 00 00 00 */
      0x24, /* 00 00 00 */
      0x18, /* 000 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 162 0xa2 '\242' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x38, /* 00 000 */
      0x54, /* 0 0 0 00 */
      0x50, /* 0 0 0000 */
      0x54, /* 0 0 0 00 */
      0x38, /* 00 000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 163 0xa3 '\243' */
      0x30, /* 00 0000 */
      0x48, /* 0 00 000 */
      0x40, /* 0 000000 */
      0x70, /* 0 0000 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x44, /* 0 000 00 */
      0x78, /* 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 164 0xa4 '\244' */
      0x44, /* 0 000 00 */
      0x24, /* 00 00 00 */
      0x50, /* 0 0 0000 */
      0x48, /* 0 00 000 */
      0x24, /* 00 00 00 */
      0x14, /* 000 0 00 */
      0x48, /* 0 00 000 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 165 0xa5 '\245' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x7c, /* 0 00 */
      0x7c, /* 0 00 */
      0x7c, /* 0 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 166 0xa6 '\246' */
      0x3c, /* 00 00 */
      0x54, /* 0 0 0 00 */
      0x54, /* 0 0 0 00 */
      0x54, /* 0 0 0 00 */
      0x3c, /* 00 00 */
      0x14, /* 000 0 00 */
      0x14, /* 000 0 00 */
      0x14, /* 000 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 167 0xa7 '\247' */
      0x18, /* 000 000 */
      0x24, /* 00 00 00 */
      0x44, /* 0 000 00 */
      0x48, /* 0 00 000 */
      0x48, /* 0 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x58, /* 0 0 000 */
      0x40, /* 0 000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 168 0xa8 '\250' */
      0x00, /* 00000000 */
      0x70, /* 0 0000 */
      0x08, /* 0000 000 */
      0x64, /* 0 00 00 */
      0x54, /* 0 0 0 00 */
      0x64, /* 0 00 00 */
      0x58, /* 0 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 169 0xa9 '\251' */
      0x00, /* 00000000 */
      0x70, /* 0 0000 */
      0x08, /* 0000 000 */
      0x34, /* 00 0 00 */
      0x44, /* 0 000 00 */
      0x34, /* 00 0 00 */
      0x08, /* 0000 000 */
      0x70, /* 0 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 170 0xaa '\252' */
      0x00, /* 00000000 */
      0x7a, /* 0 0 0 */
      0x2e, /* 00 0 0 */
      0x2e, /* 00 0 0 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 171 0xab '\253' */
      0x00, /* 00000000 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 172 0xac '\254' */
      0x00, /* 00000000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 173 0xad '\255' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x08, /* 0000 000 */
      0x7c, /* 0 00 */
      0x10, /* 000 0000 */
      0x7c, /* 0 00 */
      0x20, /* 00 00000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 174 0xae '\256' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x50, /* 0 0 0000 */
      0x50, /* 0 0 0000 */
      0x78, /* 0 000 */
      0x50, /* 0 0 0000 */
      0x50, /* 0 0 0000 */
      0x5c, /* 0 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 175 0xaf '\257' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x4c, /* 0 00 00 */
      0x54, /* 0 0 0 00 */
      0x64, /* 0 00 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 176 0xb0 '\260' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x6c, /* 0 0 00 */
      0x54, /* 0 0 0 00 */
      0x6c, /* 0 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 177 0xb1 '\261' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x7c, /* 0 00 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x7c, /* 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 178 0xb2 '\262' */
      0x00, /* 00000000 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x20, /* 00 00000 */
      0x10, /* 000 0000 */
      0x08, /* 0000 000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 179 0xb3 '\263' */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x08, /* 0000 000 */
      0x04, /* 00000 00 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x1c, /* 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 180 0xb4 '\264' */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x28, /* 00 0 000 */
      0x7c, /* 0 00 */
      0x10, /* 000 0000 */
      0x7c, /* 0 00 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 181 0xb5 '\265' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x48, /* 0 00 000 */
      0x48, /* 0 00 000 */
      0x48, /* 0 00 000 */
      0x48, /* 0 00 000 */
      0x74, /* 0 0 00 */
      0x40, /* 0 000000 */
      0x40, /* 0 000000 */
      0x00, /* 00000000 */

      /* 182 0xb6 '\266' */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x08, /* 0000 000 */
      0x0c, /* 0000 00 */
      0x14, /* 000 0 00 */
      0x24, /* 00 00 00 */
      0x24, /* 00 00 00 */
      0x18, /* 000 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 183 0xb7 '\267' */
      0x00, /* 00000000 */
      0x7c, /* 0 00 */
      0x24, /* 00 00 00 */
      0x10, /* 000 0000 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x24, /* 00 00 00 */
      0x7c, /* 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 184 0xb8 '\270' */
      0x00, /* 00000000 */
      0x7c, /* 0 00 */
      0x28, /* 00 0 000 */
      0x28, /* 00 0 000 */
      0x28, /* 00 0 000 */
      0x28, /* 00 0 000 */
      0x28, /* 00 0 000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 185 0xb9 '\271' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7c, /* 0 00 */
      0x28, /* 00 0 000 */
      0x28, /* 00 0 000 */
      0x28, /* 00 0 000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 186 0xba '\272' */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x60, /* 0 00000 */
      0x00, /* 00000000 */

      /* 187 0xbb '\273' */
      0x00, /* 00000000 */
      0x1c, /* 000 00 */
      0x24, /* 00 00 00 */
      0x24, /* 00 00 00 */
      0x1c, /* 000 00 */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 188 0xbc '\274' */
      0x00, /* 00000000 */
      0x18, /* 000 000 */
      0x24, /* 00 00 00 */
      0x24, /* 00 00 00 */
      0x18, /* 000 000 */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 189 0xbd '\275' */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x28, /* 00 0 000 */
      0x6c, /* 0 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 190 0xbe '\276' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x54, /* 0 0 0 00 */
      0x5c, /* 0 0 00 */
      0x50, /* 0 0 0000 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 191 0xbf '\277' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x4c, /* 0 00 00 */
      0x54, /* 0 0 0 00 */
      0x64, /* 0 00 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 192 0xc0 '\300' */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x20, /* 00 00000 */
      0x40, /* 0 000000 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 193 0xc1 '\301' */
      0x00, /* 00000000 */
      0x08, /* 0000 000 */
      0x00, /* 00000000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x08, /* 0000 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 194 0xc2 '\302' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7c, /* 0 00 */
      0x04, /* 00000 00 */
      0x04, /* 00000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 195 0xc3 '\303' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x0c, /* 0000 00 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x50, /* 0 0 0000 */
      0x20, /* 00 00000 */
      0x20, /* 00 00000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 196 0xc4 '\304' */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x38, /* 00 000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x60, /* 0 00000 */
      0x00, /* 00000000 */

      /* 197 0xc5 '\305' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x04, /* 00000 00 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x40, /* 0 000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 198 0xc6 '\306' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x10, /* 000 0000 */
      0x28, /* 00 0 000 */
      0x28, /* 00 0 000 */
      0x44, /* 0 000 00 */
      0x7c, /* 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 199 0xc7 '\307' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x24, /* 00 00 00 */
      0x48, /* 0 00 000 */
      0x48, /* 0 00 000 */
      0x24, /* 00 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 200 0xc8 '\310' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x48, /* 0 00 000 */
      0x24, /* 00 00 00 */
      0x24, /* 00 00 00 */
      0x48, /* 0 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 201 0xc9 '\311' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x54, /* 0 0 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 202 0xca '\312' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 203 0xcb '\313' */
      0x10, /* 000 0000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x7c, /* 0 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 204 0xcc '\314' */
      0x58, /* 0 0 000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x7c, /* 0 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 205 0xcd '\315' */
      0x58, /* 0 0 000 */
      0x38, /* 00 000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 206 0xce '\316' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x50, /* 0 0 0000 */
      0x50, /* 0 0 0000 */
      0x58, /* 0 0 000 */
      0x50, /* 0 0 0000 */
      0x50, /* 0 0 0000 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 207 0xcf '\317' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x28, /* 00 0 000 */
      0x54, /* 0 0 0 00 */
      0x5c, /* 0 0 00 */
      0x50, /* 0 0 0000 */
      0x2c, /* 00 0 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 208 0xd0 '\320' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 209 0xd1 '\321' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7e, /* 0 0 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 210 0xd2 '\322' */
      0x00, /* 00000000 */
      0x14, /* 000 0 00 */
      0x28, /* 00 0 000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 211 0xd3 '\323' */
      0x00, /* 00000000 */
      0x14, /* 000 0 00 */
      0x14, /* 000 0 00 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 212 0xd4 '\324' */
      0x00, /* 00000000 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x18, /* 000 000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 213 0xd5 '\325' */
      0x00, /* 00000000 */
      0x18, /* 000 000 */
      0x08, /* 0000 000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 214 0xd6 '\326' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x7c, /* 0 00 */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 215 0xd7 '\327' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x10, /* 000 0000 */
      0x28, /* 00 0 000 */
      0x44, /* 0 000 00 */
      0x28, /* 00 0 000 */
      0x10, /* 000 0000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 216 0xd8 '\330' */
      0x00, /* 00000000 */
      0x28, /* 00 0 000 */
      0x00, /* 00000000 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x44, /* 0 000 00 */
      0x3c, /* 00 00 */
      0x04, /* 00000 00 */
      0x38, /* 00 000 */
      0x00, /* 00000000 */

      /* 217 0xd9 '\331' */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x7e, /* 0 0 */
      0x00, /* 00000000 */
      0x7e, /* 0 0 */
      0x00, /* 00000000 */
      0x7e, /* 0 0 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 218 0xda '\332' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 219 0xdb '\333' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 220 0xdc '\334' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 221 0xdd '\335' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 222 0xde '\336' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 223 0xdf '\337' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 224 0xe0 '\340' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 225 0xe1 '\341' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 226 0xe2 '\342' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 227 0xe3 '\343' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 228 0xe4 '\344' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 229 0xe5 '\345' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 230 0xe6 '\346' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 231 0xe7 '\347' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 232 0xe8 '\350' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 233 0xe9 '\351' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 234 0xea '\352' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 235 0xeb '\353' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 236 0xec '\354' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 237 0xed '\355' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 238 0xee '\356' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 239 0xef '\357' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 240 0xf0 '\360' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 241 0xf1 '\361' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 242 0xf2 '\362' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 243 0xf3 '\363' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 244 0xf4 '\364' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 245 0xf5 '\365' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 246 0xf6 '\366' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 247 0xf7 '\367' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 248 0xf8 '\370' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 249 0xf9 '\371' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 250 0xfa '\372' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 251 0xfb '\373' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 252 0xfc '\374' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 253 0xfd '\375' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 254 0xfe '\376' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */

      /* 255 0xff '\377' */
      0x00, /* 00000000 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x3c, /* 00 00 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
      0x00, /* 00000000 */
   };

}
